// Type definitions for ./src/common/signals.js
// Project: [LIBRARY_URL_HERE] 
// Definitions by: [YOUR_NAME_HERE] <[YOUR_URL_HERE]> 
// Definitions: https://github.com/borisyankov/DefinitelyTyped
declare namespace signals.prototype{
	// signals.prototype._registerListener.!ret
	
	/**
	 * 
	 */
	interface _registerListenerRet {
				
		/**
		 * If binding should be executed just once.
		 * @type boolean
		 * @private
		 */
		_isOnce : boolean;
				
		/**
		 * Reference to Signal object that listener is currently bound to.
		 * @type Signal
		 * @private
		 */
		_signal : signals;
				
		/**
		 * Listener priority
		 * @type Number
		 * @private
		 */
		_priority : number;
				
		/**
		 * 
		 */
		_listener : Function;
				
		/**
		 * 
		 */
		context : any;
	}
}

/**
 * Custom event broadcaster
 * <br />- inspired by Robert Penner's AS3 Signals.
 * @name Signal
 * @author Miller Medeiros
 * @constructor
 */
declare interface signals {
		
	/**
	 * 
	 */
	new ();
		
	/**
	 * Signals Version Number
	 * @type String
	 * @const
	 */
	VERSION : string;
		
	/**
	 * If Signal should keep record of previously dispatched parameters and
	 * automatically execute listener during `add()`/`addOnce()` if Signal was
	 * already dispatched before.
	 * @type boolean
	 */
	memorize : boolean;
		
	/**
	 * 
	 */
	_shouldPropagate : boolean;
		
	/**
	 * If Signal is active and should broadcast events.
	 * <p><strong>IMPORTANT:</strong> Setting this property during a dispatch will only affect the next dispatch, if you want to stop the propagation of a signal use `halt()` instead.</p>
	 * @type boolean
	 */
	active : boolean;
		
	/**
	 * @param {Function} listener
	 * @param {boolean} isOnce
	 * @param {Object} [listenerContext]
	 * @param {Number} [priority]
	 * @return {SignalBinding}
	 * @private
	 * @param listener 
	 * @param isOnce 
	 * @param listenerContext? 
	 * @param priority? 
	 * @return  
	 */
	_registerListener(listener : Function, isOnce : boolean, listenerContext? : any, priority? : number): /* signals.prototype._registerListener.!ret */ any;
		
	/**
	 * @param {SignalBinding} binding
	 * @private
	 * @param binding 
	 */
	_addBinding(binding : /* signals.prototype._registerListener.!ret */ any): void;
		
	/**
	 * @param {Function} listener
	 * @return {number}
	 * @private
	 * @param listener 
	 * @param context 
	 * @return  
	 */
	_indexOfListener(listener : Function, context : any): number;
		
	/**
	 * Check if listener was attached to Signal.
	 * @param {Function} listener
	 * @param {Object} [context]
	 * @return {boolean} if Signal has the specified listener.
	 * @param listener 
	 * @param context? 
	 * @return  
	 */
	has(listener : Function, context? : any): boolean;
		
	/**
	 * Add a listener to the signal.
	 * @param {Function} listener Signal handler function.
	 * @param {Object} [listenerContext] Context on which listener will be executed (object that should represent the `this` variable inside listener function).
	 * @param {Number} [priority] The priority level of the event listener. Listeners with higher priority will be executed before listeners with lower priority. Listeners with same priority level will be executed at the same order as they were added. (default = 0)
	 * @return {SignalBinding} An Object representing the binding between the Signal and listener.
	 * @param listener 
	 * @param listenerContext? 
	 * @param priority? 
	 * @return  
	 */
	add(listener : Function, listenerContext? : any, priority? : number): /* signals.prototype._registerListener.!ret */ any;
		
	/**
	 * Add listener to the signal that should be removed after first execution (will be executed only once).
	 * @param {Function} listener Signal handler function.
	 * @param {Object} [listenerContext] Context on which listener will be executed (object that should represent the `this` variable inside listener function).
	 * @param {Number} [priority] The priority level of the event listener. Listeners with higher priority will be executed before listeners with lower priority. Listeners with same priority level will be executed at the same order as they were added. (default = 0)
	 * @return {SignalBinding} An Object representing the binding between the Signal and listener.
	 * @param listener 
	 * @param listenerContext? 
	 * @param priority? 
	 * @return  
	 */
	addOnce(listener : Function, listenerContext? : any, priority? : number): /* signals.prototype._registerListener.!ret */ any;
		
	/**
	 * Remove a single listener from the dispatch queue.
	 * @param {Function} listener Handler function that should be removed.
	 * @param {Object} [context] Execution context (since you can add the same handler multiple times if executing in a different context).
	 * @return {Function} Listener handler function.
	 * @param listener 
	 * @param context? 
	 * @return  
	 */
	remove(listener : Function, context? : any): Function;
		
	/**
	 * Remove all listeners from the Signal.
	 */
	removeAll(): void;
		
	/**
	 * @return {number} Number of listeners attached to the Signal.
	 * @return  
	 */
	getNumListeners(): /* !this._bindings.length */ any;
		
	/**
	 * Stop propagation of the event, blocking the dispatch to next listeners on the queue.
	 * <p><strong>IMPORTANT:</strong> should be called only during signal dispatch, calling it before/after dispatch won't affect signal broadcast.</p>
	 * @see Signal.prototype.disable
	 */
	halt(): void;
		
	/**
	 * 
	 */
	dispatch(): void;
		
	/**
	 * Forget memorized arguments.
	 * @see Signal.memorize
	 */
	forget(): void;
		
	/**
	 * Remove all bindings from signal and destroy any reference to external objects (destroy Signal object).
	 * <p><strong>IMPORTANT:</strong> calling any method on the signal instance after calling dispose will throw errors.</p>
	 */
	dispose(): void;
		
	/**
	 * 
	 */
	_prevParams : Array<any>;
		
	/**
	 * @type Array.<SignalBinding>
	 * @private
	 */
	_bindings : Array<any>;
}
