var jccloud = require('./service/jccloud');
var jcgamelog = require('./service/jcgamelog');
var jclogin = require('./service/jclogin');
var jcshare = require('./service/jcshare');
var jcAD = require('./service/jcAD');
var jcStat = require('./service/jcstat');
var jcNotify = require('./service/jcnotify');
var jcvoodoo = require('./service/jcvoodoo');
var jcmail = require('./service/jcmail');
var jcadshare = require('./service/jcadsharectrl');
var jclog = require('./service/jclog');
var jcrecharge = require('./service/jcrecharge');
var jcrank = require('./service/jcrank');
var jcevent = require('./jc-event');
var jcgoods = require('./service/jcgoods');
var jcfcm = require('./fcm/jcfcm');

var storage = require('./common/storage');
var storageutils = require('./utils/storageutils');
var strutils = require('./utils/strutils');

const notification = require('./common/notification');

const _GAME_LOCALUUID = 'jc_game_localuuid';

const _GAME_INFO_HANDLER_BINDING = 'jc_gameinfo_handler_binding';
const _GAME_INFO_HANDLER_UNBINDING = 'jc_gameinfo_handler_unbinding';

const JC_CLOUD_URL = 'cloud';
const JC_GAMELOG_URL = 'gamelog';
const JC_LOGIN_URL = 'login';
const JC_NOTIFY_URL = 'notify';
const JC_SVC_URL = 'service';
const JC_STAT_URL = 'stat';
const JC_GAMELIST_URL = 'service';
const JC_MAIL_URL = 'gamemail';
const JS_RECHARGE = 'game1008-pay';
const JC_GHOST_URL = 'ghost';

const KINGSOME = 'kingsome.cn';
const KSOME = 'ksome.cn';

cc.gg = cc.gg || {};

var JCFW = {
	cloud: jccloud,
	gamelog: jcgamelog, // 打点用对象
	login: jclogin, // 登陆用对象
	share: jcshare, // 邀请相关服务
	ad: jcAD, // 广告对象
	stat: jcStat,
	notify: jcNotify,
	voodoo: jcvoodoo,
	mail: jcmail, // 邮件用对象
	recharge: jcrecharge,
	rank: jcrank, // 排行榜对象
	goods: jcgoods, // 实物兑换
	adshare: jcadshare, // 分享/广告控制用对象
	cclog: jclog,
	ccstorage: storageutils, // 存储用对象（支持云存储和本地存储）

	event: jcevent, // jc通用活动系统通知事件

	fcm: jcfcm,

	plat: null, // 平台相关API

	localUUID: '',

	systemInfo: null,
	lauchInfo: null,
	shareInfo: {},
	_handlers: [],

	_inited: false,
	_dayvideocount: -1,

	_greentime: 1.5,

	version: '',

	channel: {
		WECHAT: 6001,
		QQ_PLAY: 6002,
		OPPO: 6003,
		VIVO: 6004,
		FB: 6501,
		BAIDU: 6005,
		TT: 6006,
		QQ_MINI: 6007,
		QUICK: 8001,
		TEST: 6000,
		ANDROID: 6502,
		GOOGLE: 6511,
	},

	isAbroad(cid){
		let id = cid? cid: this.channelID;
		return id == this.channel.GOOGLE;
	},

	__getUrl(isoffical, key, env) {
		let pre = key;
		if (!isoffical) {
			pre += '-test';
		}
		let dm = KINGSOME;
		// 现在只有cloud和login的正式服有海外服务器
		if (isoffical && (key === JC_CLOUD_URL || key === JC_LOGIN_URL || key === JC_GAMELOG_URL)) {
			dm = this.isAbroad()? KSOME: KINGSOME;
		}

		let str = 'https://' + pre + '.' + dm + '/webapp/index.php';

		if (env) {
			str += '?publish_env=dev';
		}
		// let str = 'https://www.xmjl123.cn/' + pre + '/webapp/index.php';
		return str;
	},

	// 初始化
	/**
	 *
	 * @param {number} channelid ： 频道ID
	 * @param {number} gameid ：游戏ID
	 * @param {boolean} isoffical ：上线版本还是测试版本
	 * @param {object} response ：回调对象（可不填），如果填的话，可实现以下一个或多个函数：
	 * .onLogin(logininfo)
	 * .onLoginFail(net_err, logic_err, errmsg)
	 * .onUpdateUser(userinfo)
	 * .onUpdateUserFail(net_err, logic_err, errmsg)
	 * .onShare(***)
	 * .onShareFail(***)
	 * @param {any} env: 某些平台登陆需要传递的参数
	 * @param {any} adparam: 某些平台广告环境需要传递的参数
	 * @param {object} cfgsuccesscb: 后台配置成功回调
	 * @param {object} cfgfailcb: 后台配置失败回调
	 * @param {any} jcdev: 是否是开发环境
	 */
	init(channelid, gameid, isoffical, response, env, adparam, cfgsuccesscb, cfgfailcb, jcdev, polyid) {
		if (!this.sharecfg) {
			this.sharecfg = {};
		}

		if (this._inited) {
			if (this.rsp != response) {
				this.rsp = response;
			}
			this._loadConfig(cfgsuccesscb, cfgfailcb);
			return;
		}
		strutils.initStringPrototype();

		this.rsp = response;

		this.gameID = parseInt(gameid);

		this.isOffical = isoffical;

		this._fixChannelID(channelid, polyid);

		this._checkListener();

		this.gamelog.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_GAMELOG_URL, jcdev));
		this.cloud.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_CLOUD_URL, jcdev));
		this.login.init(this.channelID, this.gameID, isoffical, this, env, this.__getUrl(isoffical, JC_LOGIN_URL, jcdev), this.polyID);
		this.share.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_SVC_URL, jcdev));
		this.ad.init(this.channelID, this.gameID, isoffical, this, adparam);
		this.stat.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_STAT_URL, jcdev));
		this.notify.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_NOTIFY_URL, jcdev));
		this.voodoo.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_GAMELIST_URL, jcdev));
		this.mail.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_MAIL_URL, jcdev));
		this.recharge.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JS_RECHARGE, jcdev));
		this.rank.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_GHOST_URL));
		this.goods.init(this.channelID, this.gameID, isoffical, this, this.__getUrl(isoffical, JC_GHOST_URL));
		this.adshare.init(this, isoffical);
		this.cclog.init(this, isoffical);
		this.ccstorage.init(this, isoffical, this.cloud);

		this._loadLocalID();
		this._loadLaunchInfo();
		this._loadSystemInfo();
		this._loadConfig(cfgsuccesscb, cfgfailcb);

		this._handleInit(this.channelID, this.gameID, isoffical);

		this._inited = true;
	},

	// 绑定游戏主场景【游戏主场景初始化时必须手动调用此函数】
	/**
	 *
	 * @param {object} man : 游戏主场景canvas的component
	 */
	bindController(man) {
		this.plat && this.plat.bindController(man);
	},

	__handleLogined() {
		if (this.cacheuser) {
			this.updateUser(this.cacheuser.info, this.cacheuser.success, this.cacheuser.fail);
		} else {
			// 海外特殊处理
			if(this.isAbroad()){
				this.sharecfg.info = {};
			}
			this.initShareConfig();
		}
	},

	/**
	 * 登陆函数
	 * @param {object} successcb ：成功回调（初始化时设置了回调对象的话，此处可不填）
	 * @param {object} failcb ：失败回调（初始化时设置了回调对象的话，此处可不填）
	 * @param {string} ex_param ：额外参数（预留字段，某些平台可能会使用此字段）
	 * @param {object} cfgsuccesscb ：配置拉取成功回调
	 * @param {object} cfgfailcb ：配置拉取失败回调
	 * @param {object} timecb ：登陆完成后的在线时长回调（大概每10秒回调一次，参数为在线时长）
	 */
	loginUser(successcb, failcb, ex_param, cfgsuccesscb, cfgfailcb, timecb) {
		console.log('loginUser');
		if (successcb) {
			this._login_successcb = successcb;
		}
		if (failcb) {
			this._login_failcb = failcb;
		}
		if (cfgsuccesscb) {
			this.sharecfg.successcb = cfgsuccesscb;
		}
		if (cfgfailcb) {
			this.sharecfg.failcb = cfgfailcb;
		}
		if (this.logined) {
			this._login_successcb && this._login_successcb(this.accountobj);
			this.__handleLogined();
			return 0;
		}
		if (this._logining) {
			return 1;
		}
		this._logining = true;
		this.login.loginPT((res) => {
			this._logining = false;
			this.logined = true;

			this.setAccountID(res.account_id, res.session_id, res);
			this.rank.setAvatar(res.avatar_url);
			this.fcm.setAge(res.age);
			// this.rank.setNickName(res.nickname);
			this.setNickName(res.nickname);
			this.setServerTime(res.server_time);
			this._isholiday = res.is_holiday? res.is_holiday: false;
			this._greentime = res.green_time? res.green_time: 1.5;
			this.gamelog.logLoginSuccess(res, this.login.login_costtime);
			this.gamelog.logSystemInfo(this.systemInfo);

			if (this.shareInfo && this.shareInfo.type && this.shareInfo.param) {
				this.share.acceptDailyInvite(this.shareInfo.param, this.shareInfo.inviter, () => {}, () => {});
			}
			if (this.shareInfo && this.shareInfo.inviter) {
				this.share.acceptAchivementInvite([
					[1001, 100]
				], this.shareInfo.inviter);
				this.gamelog.logShareInvite(this.shareInfo.inviter, this.shareInfo.type, this.shareInfo.id, this.shareInfo.param);
			}

			this.rsp && this.rsp.onLogin && this.rsp.onLogin(res);
			this._login_successcb && this._login_successcb(res);
			this.__handleLogined();
			this.adshare.loginOver();

		}, (neterr, logicerr, errmsg) => {
			this._logining = false;
			this.gamelog.logLoginFailed(neterr, logicerr, errmsg);
			this.rsp && this.rsp.onLoginFail && this.rsp.onLoginFail(neterr, logicerr, errmsg);
			this._login_failcb && this._login_failcb(neterr, logicerr, errmsg);
			this.sharecfg.failcb && this.sharecfg.failcb(neterr, logicerr, errmsg);
		}, ex_param, (game_time)=>{
			this.fcm.onLineTimeCb(game_time, timecb);
		});
		return 0;
	},

	/**
	 *
	 * @param {string} token : 登陆用token
	 * @param {object} successcb : 成功后回调
	 * @param {object} failcb : 失败后回调
	 */
	loginUserByToken(token, successcb, failcb) {
		console.log('loginUserByToken:' + token);
		return this.loginUser(successcb, failcb, token);
	},

	logout: function () {
		this.logined = false;
		this.login.clear();
		this.playerdata && (this.playerdata.info = null);
		this.fcm.offLine();
	},

	// 激活
	/**
	 *
	 * @param {object} allinfo ：包含用户信息的对象: 注意：allinfo.userInfo为用户实际信息
	 * @param {object} successcb ：成功回调（初始化时设置了回调对象的话，此处可不填）
	 * @param {object} failcb ：失败回调（初始化时设置了回调对象的话，此处可不填）
	 */
	updateUser(allinfo, successcb, failcb) {
		console.log('updateUser');
		if (!this.logined) {
			if (!this.cacheuser) {
				this.cacheuser = {};
			}
			this.cacheuser.info = allinfo;
			this.cacheuser.success = successcb;
			this.cacheuser.fail = failcb;
			return;
		} else {
			this.cacheuser = null;
		}
		let userinfo = allinfo && allinfo.userInfo;
		if (!userinfo) {
			this.gamelog.logAuthFail();
			this.setUserInfo(null);
		} else {
			this.gamelog.logAuthSuccess(userinfo);
			this.setUserInfo(userinfo);
		}

		if (!allinfo || !userinfo) {
			this.rsp && this.rsp.onUpdateUserFail && this.rsp.onUpdateUserFail(0, -10000, 'info is NULL');
			failcb && failcb(0, -10000, 'info is NULL');
			return 0;
		}

		if (this._userupdating) {
			return 1;
		}
		this._userupdating = true;
		this.login.updatePTInfo(userinfo, allinfo, (obj) => {
			this._userupdating = false;
			this.rsp && this.rsp.onUpdateUser && this.rsp.onUpdateUser(obj);
			successcb && successcb(obj);
		}, (neterr, logicerr, errmsg) => {
			this._userupdating = false;
			this.rsp && this.rsp.onUpdateUserFail && this.rsp.onUpdateUserFail(neterr, logicerr, errmsg);
			failcb && failcb(neterr, logicerr, errmsg);
		});
		return 0;
	},

	// 绑定身份证
	/**
     *
     * @param {string} name : 真实姓名
     * @param {string} idcard : 真实身份证号
     * @param {boolean} isfake : 是否是假的（不校验）
     * @param {object} successcb : 成功回调，参数为玩家年龄
     * @param {object} failcb : 失败回调
     */
    bindIdCard(name, idcard, isfake, successcb, failcb){
		this.login.bindIdCard(name, idcard, isfake, (age)=>{
			this.fcm.setAge(age);
			successcb && successcb(age);
		}, failcb);
	},

	// 获取用户信息
	/**
	 *
	 * @param {object} btninfo : 获取用户的按钮信息（微信专用），格式：{width: 100, height: 100, name: logo.png}
	 * @param {object} successcb ：成功回调
	 * @param {object} failcb ：失败回调
	 */
	getUserInfo(btninfo, successcb, failcb) {
		this.plat && this.plat.getUserInfo && this.plat.getUserInfo(btninfo, (res, err, exobj) => {
			if (err < 0) {
				failcb && failcb(0, err, 'getuserinfo err');
			} else if (err > 0) {
				this.updateUser(res, successcb, failcb);
			} else {
				successcb && successcb(null, exobj); //注意：微信专用，创建获取用户按钮后回调
			}
		})
	},

	// 清除授权按钮（微信用）
	destoryAuthComponent() {
		this.plat && this.plat.destoryAuth && this.plat.destoryAuth();
	},

	// 获取基本配置信息
	/**
	 *
	 * @param {object} successcb ：成功回调
	 * @param {object} failcb ：失败回调
	 */
	initGlobalConfig(successcb, failcb) {
		if (!this.maincfg) {
			this.maincfg = {};
		}
		if (this.maincfg.info) {
			successcb && successcb(this.maincfg.info);
			return;
		}
		if (successcb) {
			this.maincfg.successcb = successcb;
		}
		if (failcb) {
			this.maincfg.failcb = failcb;
		}
		this.cloud.getServerConfig((cfg) => {
			this._handleCfg(cfg);
			this.rsp && this.rsp.onServerConfig && this.rsp.onServerConfig(cfg);
			this.maincfg.successcb && this.maincfg.successcb(cfg);
		}, (neterr, logicerr, errmsg) => {
			this.rsp && this.rsp.onServerConfigFail && this.rsp.onServerConfigFail(neterr, logicerr, errmsg);
			this.maincfg.failcb && this.maincfg.failcb(neterr, logicerr, errmsg);
		});
	},

	// 获取分享相关配置信息
	/**
	 *
	 * @param {object} successcb ：成功回调
	 * @param {object} failcb ：失败回调
	 */
	initShareConfig(successcb, failcb, bforce) {
		if (successcb) {
			this.sharecfg.successcb = successcb;
		}
		if (failcb) {
			this.sharecfg.failcb = failcb;
		}
		if (this.sharecfg.info && !bforce) {
			this.sharecfg.successcb && this.sharecfg.successcb(this.sharecfg.info);
			return;
		}
		this.cloud.initShareConfig((cfg) => {
			this.adshare.loadShareConfig && this.adshare.loadShareConfig(cfg);
			this.plat && this.plat.PTInitShare && this.plat.PTInitShare(this.adshare._commonshareinfo);

			this.rsp && this.rsp.onShareConfig && this.rsp.onShareConfig(cfg);
			this.sharecfg.info = cfg;
			cc.gg.sharecfg = cfg;
			this.sharecfg.successcb && this.sharecfg.successcb(cfg);
		}, (neterr, logicerr, errmsg) => {
			this.rsp && this.rsp.onShareConfigFail && this.rsp.onShareConfigFail(neterr, logicerr, errmsg);
			this.sharecfg.failcb && this.sharecfg.failcb(neterr, logicerr, errmsg);
		}, bforce);
	},

	// 获取用户相关配置信息
	/**
	 *
	 * @param {object} successcb ：成功回调
	 * @param {object} failcb ：失败回调
	 */
	initPlayerData(successcb, failcb) {
		if (!this.playerdata) {
			this.playerdata = {};
		}
		if (this.playerdata.info) {
			successcb && successcb(this.playerdata.info);
			return;
		}
		if (successcb) {
			this.playerdata.successcb = successcb;
		}
		if (failcb) {
			this.playerdata.failcb = failcb;
		}
		if (this.playergetting) {
			return;
		}
		this.playergetting = true;
		this.cloud.getBigData((cfg) => {
			this.playergetting = false;
			if (cfg) {
				this.playerdata.info = cfg;
			} else if (!this.playerdata.info) {
				this.playerdata.info = {};
			}
			cc.gg.playerdata = this.playerdata.info;
			this.rsp && this.rsp.onPlayerData && this.rsp.onPlayerData(this.playerdata.info);
			this.playerdata.successcb && this.playerdata.successcb(this.playerdata.info);
		}, (neterr, logicerr, errmsg) => {
			this.playergetting = false;
			this.rsp && this.rsp.onPlayerDataFail && this.rsp.onPlayerDataFail(neterr, logicerr, errmsg);
			this.playerdata.failcb && this.playerdata.failcb(neterr, logicerr, errmsg);
		});
	},

	// 保存用户相关配置信息
	savePlayerData(successcb, failcb) {
		if (this.playersetting) {
			this._needsavedata = true;
			failcb && failcb(-12345);
			return;
		}
		this.playersetting = true;
		this._needsavedata = false;
		this.cloud.setBigData(this.playerdata.info, ()=>{
			this.playersetting = false;
			this._retrysavedata = 0;
			successcb && successcb();
			if(this._needsavedata){
				this.savePlayerData();
			}
		}, ()=>{
			this.playersetting = false;
			failcb && failcb();
			if(!this._retrysavedata){
				this._retrysavedata = 1;
			}else{
				this._retrysavedata++;
			}
			if(this._retrysavedata <= 3){
				this.savePlayerData();
			}
		});
	},

	// 简化接收每日任务/成就邀请(默认1001为邀请好友的成就，其他成就不可使用此id)
	acceptInvite(info) {
		if (info.daily) {
			this.share.acceptDailyInvite(info.daily.id, this.shareInfo.inviter);
		}
		if (info.achivement) {
			this.share.acceptAchivementInvite(info.achivement.idlist, this.shareInfo.inviter);
		}
	},

	/**
	 * 普通分享
	 * @param {string} content 分享显示的标题
	 * @param {string} imgurl 分享显示的图片
	 * @param {int} sharetype 分享类型（每日活动/成就/接力）
	 * @param {string} shareparam 分享类型相关参数（活动id/成就id等）
	 * @param {string} extrainfo 分享额外参数（key/value格式的字符串：eg: 'inviter_name=***&inviter_sex=***'
	 * @param {object} successcb
	 * @param {object} failcb
	 * @param {number} imgindex 图片在后台的索引（可不填）
	 */
	shareNormal(content, imgurl, sharetype, shareparam, extrainfo, successcb, failcb, imgindex) {
		if (this.plat && this.plat.commonShare) {
			this.plat.commonShare(imgurl, content, sharetype, shareparam, (param, tokenid) => {
				this.gamelog.logShare(sharetype, tokenid, param, null, imgindex);
				this.rsp && this.rsp.onShare && this.rsp.onShare(content, imgurl, sharetype, shareparam, extrainfo);
				successcb && successcb();
			}, (err, param, tokenid) => {
				let serr = '' + err;
				this.gamelog.logShare(sharetype, tokenid, param, serr, imgindex);
				this.rsp && this.rsp.onShareFail && this.rsp.onShareFail(content, imgurl, sharetype, shareparam, extrainfo);
				failcb && failcb(err);
			}, extrainfo);
		} else {
			failcb && failcb(-1000);
		}
	},

	/**
	 * 截图分享
	 * @param {string} content 分享显示的标题
	 * @param {cc.Rect} capture_rect 分享要截图的区域
	 * @param {int} sharetype 分享类型（每日活动/成就/接力）
	 * @param {string} shareparam 分享类型相关参数（活动id/成就id等）
	 * @param {string} extrainfo 分享额外参数（key/value格式的字符串：eg: 'inviter_name=***&inviter_sex=***'
	 * @param {object} successcb
	 * @param {object} failcb
	 */
	shareCapture(content, capture_rect, sharetype, shareparam, extrainfo, successcb, failcb, imgindex) {
		if (this.plat && this.plat.captureShare) {
			this.plat.captureShare(capture_rect, content, sharetype, shareparam, (param, tokenid, dtime) => {
				this.gamelog.logShare(sharetype, tokenid, param, null, imgindex, dtime);
				this.rsp && this.rsp.onShare && this.rsp.onShare(content, capture_rect, sharetype, shareparam, extrainfo);
				successcb && successcb();
			}, (err, param, tokenid, dtime) => {
				let serr = '' + err;
				this.gamelog.logShare(sharetype, tokenid, param, serr, imgindex, dtime);
				this.rsp && this.rsp.onShareFail && this.rsp.onShareFail(content, capture_rect, sharetype, shareparam, extrainfo);
				failcb && failcb(err);
			}, extrainfo);
		} else {
			failcb && failcb(-1000);
		}
	},

	// 生成分享参数字符串
	makeShareParam(sharetype, shareparam, extrainfo) {
		let str = '';
		if (typeof (shareparam) == 'string') {
			str = shareparam;
		} else if (typeof (shareparam) == 'object') {
			str = JSON.stringify(shareparam);
		}
		this._tmpuuid = this.makeUUID();
		let qstr = "inviter_id=" + this.accountID + "&activity_param=" +
			str + '&localuuid=' + this._tmpuuid + '&sharetype=' + sharetype;
		if (extrainfo) {
			qstr += '&';
			qstr += extrainfo;
		}
		return qstr;
	},

	//根据配置处理普通分享
	/**
	 *
	 * @param {object} successcb	成功回调
	 * @param {object} failcb 		失败回调
	 * @param {string} typestr 		分享类型【与后台配置相符】
	 * @param {string} exparam 		额外参数
	 */
	simpleShare(successcb, failcb, typestr, exparam) {
		let stype = typestr ? typestr : 'normal';
		let info = this.adshare.getShareInfo(stype);
		if (this.plat && this.plat.commonShare) {
			this.plat.commonShare(info.imgurl, info.title, stype, exparam, (param, tokenid, dtime) => {
					this.gamelog.logShare(stype, tokenid, param, null, info.imgindex, dtime);
					this.rsp && this.rsp.onShare && this.rsp.onShare(info.title, info.imgurl, stype, param, exparam);
					successcb && successcb();
				},
				(err, param, tokenid, dtime) => {
					let serr = '' + err;
					this.gamelog.logShare(stype, tokenid, param, serr, info.imgindex, dtime);
					this.rsp && this.rsp.onShareFail && this.rsp.onShareFail(info.title, info.imgurl, stype, param, exparam);
					failcb && failcb(err);
				});
		}
	},

	//根据配置决定是分享还是广告（广告转分享用）
	/**
	 *
	 * @param {string} stype	分享/广告类型【与后台配置相符】
	 * @param {string} param 	额外参数(必须是json格式):
	 * 							a)如果需要截屏分享，参数需配置成如下格式：
	 * 								let paramobj = {
	 * 									...
	 * 									rc:{x:*,y:*,width:**,height:**},
	 * 									...
	 * 								};
	 * 							b)如果分享需要指定分享组的某张图片，参数需配置成如下格式：
	 * 								let paramobj = {
	 * 									...
	 * 									shareindex: **(分享组图片索引)
	 * 									...
	 * 								};
	 * 						  	c)如果分享需要动态的title，参数需配置成如下格式：
	 * 						  		let paramobj = {
	 * 									...
	 * 									titleparam: [*,*,...](title参数)
	 * 									...
	 * 								};
	 * @param {object} cb 		回调，参数如下：
	 * 										【参数1】isOK:是否成功：false/true，
	 * 										【参数2】errCode:失败的errcode,
	 * 										【参数3】flag:是分享还是广告的标识：0-none;1-share;2-ad,
	 * 										【参数4】isADErr:广告失败时，是否是出错了（false：未看完关闭/true：出错）
	 * 										【参数5】isDiff:最终是分享（广告），是否与后台配置的一致
	 */
	ADShare(stype, param, cb) {
		if (this._istestmode) {
			cb && cb(true, 0, 0);
			return;
		}
		return this.adshare.AD_Share(stype, param, (isok, errcode, flag, isaderr, isdiff) => {
			cb && cb(isok, errcode, flag, isaderr, isdiff);
			if (flag == 2 && !isaderr) {
				let data = {
					isOK: isok,
					errCode: errcode
				};
				this.event.emit(this.event.Events.AD_VIDEO_CLOSED, data);
			}
		});
	},
	/**
	 * 优先观看视频广告，到指定次数后分享，分享内容读取配置
	 * @param {string} stype	分享/广告类型【与后台配置相符】
	 * @param {string} param 	额外参数(必须是json格式)
	 * @param {object} cb 		回调，参数：1:是否成功：false/true，2.失败的errcode, 3.是分享还是广告的标识：0-none;1-share;2-ad
	 * */
	ADShareCommon(stype, param, cb) {
		if (cc.jc.channelID === cc.jc.channel.TEST) {
			cb && cb(true, 0, 0);
			return;
		}
		if (this._istestmode) {
			cb && cb(true, 0, 0);
			return;
		}
		return this.adshare.AD_Share_Common(stype, param, cb);
	},

	// 激励视频广告【尽量使用ADShare函数而不使用此函数】
	ADWatch(adid, adname, successcb, failcb) {
		console.log(`play video ad: ${adid}`);
		let sname = adname ? adname : adid;
		if (this._istestmode) {
			this.rsp && this.rsp.onADFinish && this.rsp.onADFinish(sname);
			successcb && successcb();
			return;
		}
		if (this._offad) {
			failcb && failcb(null, true);
			return;
		}
		this.ad.showRewardAD(sname, (currid, res, realadid) => {
			this.gamelog.logAdvInfo(realadid, sname);
			this.rsp && this.rsp.onADFinish && this.rsp.onADFinish(sname);
			successcb && successcb();
		}, (currid, res, iserr, realadid) => {
			if (res) {
				this.gamelog.logAdvInfo(realadid, sname, res);
			}
			this.rsp && this.rsp.onADFail && this.rsp.onADFail(sname, res);
			failcb && failcb(res, iserr);
		}, (currid, res, realadid) => {
			this.gamelog.logAdvInfo(realadid, sname, null, 0, 1);
		}, adid);
	},

	// 激励视频广告（可多个视频id随机播放）【尽量使用ADShare函数而不使用此函数】
	ADsWatch(adlst, adname, successcb, failcb) {
		if (this._istestmode) {
			this.rsp && this.rsp.onADFinish && this.rsp.onADFinish(adname);
			successcb && successcb();
			return;
		}
		if (this._offad) {
			failcb && failcb(null, true);
			return;
		}
		this.ad.randomRewardADShow(adlst, adname, (currid, res, realadid) => {
			let sname = adname ? adname : realadid;
			this.gamelog.logAdvInfo(realadid, sname);
			this.rsp && this.rsp.onADFinish && this.rsp.onADFinish(sname);
			successcb && successcb();
		}, (currid, res, iserr, realadid) => {
			let sname = adname ? adname : realadid;
			if (res) {
				this.gamelog.logAdvInfo(realadid, sname, res);
			}
			this.rsp && this.rsp.onADFail && this.rsp.onADFail(sname, res);
			failcb && failcb(res, iserr);
		}, (currid, res, realadid) => {
			this.gamelog.logAdvInfo(realadid, sname, null, 0, 1);
		});
	},

	//banner处理【尽量使用Baselayer而不使用以下banner处理函数】
	// banner初始化
	ADBanner_init(adid, adname) {
		if (this._offbanner) {
			return;
		}

		if (!adid) {
			adid = this._bannerids;
		}
		if (!adid) {
			return false;
		}
		if (adid instanceof Array) {
			adid.forEach(element => {
				if (!this.ad.hasBannerAD(element)) {
					this.ad.addBannerAD(element, element);
				}
			});
		} else {
			let name = adname ? adname : adid;
			if (!this.ad.hasBannerAD(name)) {
				this.ad.addBannerAD(adid, name);
			}
		}
		return true;
	},

	// banner显示
	ADBanner_Show(adname, successcb, failcb, adid, breload) {
		if (this._offbanner) {
			return '';
		}
		let id = adid;
		if (!id) {
			id = this._randomBannerAD();
		}
		this.ad.switchBannerAD(id, true, () => {
			this.gamelog.logAdvInfo(id, adname, null, 1);
		}, (name, err, advid) => {
			this.gamelog.logAdvInfo(id, adname, err, 1);
		}, breload);
		return id;
	},

	// banner隐藏
	ADBanner_Hide(adname) {
		if (this._offbanner) {
			return;
		}
		this.ad.switchBannerAD(adname, false);
	},

	// banner直接显示（无需初始化）
	ADBanner_ShowEx(adid, adname, successcb, failcb, adwidth) {
		if (this._offbanner) {
			return null;
		}
		let id = adid;
		if (!id) {
			id = this._randomBannerAD();
		}
		return this.ad.showBannerAD(id, () => {
			this.gamelog.logAdvInfo(id, adname, null, 1);
			successcb && successcb();
		}, (name, err, advid) => {
			this.gamelog.logAdvInfo(id, adname, err, 1);
			failcb && failcb();
		}, () => {
			this.gamelog.logAdvInfo(id, adname, null, 1, 1);
		}, adwidth, this._bannertimeout);
	},

	// banner销毁
	ADBanner_Destroy(obj, realdestroy) {
		try {
			if (realdestroy) {
				this.ad.destroyObj(obj, true);
			} else {
				this.ad.hideAD(obj);
			}
		} catch (err) {

		}
	},

	// 手动控制banner的显示/隐藏
	ADBanner_Switch(bshow) {
		bshow ? this.event.emit(this.event.Events.BANNER_MANUAL_SHOW) : this.event.emit(this.event.Events.BANNER_MANUAL_HIDE);
	},

	//插屏广告处理
	ADInsert_Show(adid, adname, successcb, failcb) {
		if (this._offinsert) {
			return;
		}
		if (!Object.values) {
			failcb && failcb(adname, null, true, adid);
			return;
		}
		let id = this.ad.simpleInsertADShow(adid, successcb, failcb);
		if (id) {
			this.gamelog.logAdvInfo(id, adname, null, 2);
		}
	},

	_randomBannerAD() {
		if (this._bannerids.length == 0) {
			return null;
		}
		if (this._bannerids.length == 1) {
			this._last_rand_ad = this._bannerids[0];
			return this._bannerids[0];
		}
		let lst = null;
		if (this._last_rand_ad) {
			lst = [];
			for (let i = 0; i < this._bannerids.length; i++) {
				if (this._bannerids[i] != this._last_rand_ad) {
					lst.push(this._bannerids[i]);
				}
			}
		} else {
			lst = this._bannerids;
		}
		let n = Math.floor(Math.random() * lst.length);
		this._last_rand_ad = lst[n];
		return this._last_rand_ad;
	},

	// 生成唯一ID【无需关注此函数】
	currUUID() {
		return this._tmpuuid;
	},
	makeUUID() {
		return this.gamelog.generateUUID();
	},
	// 根据OpenID生成账号ID【通常情况不使用此函数】
	makeAccountID(openid) {
		return this.login.AccountID(openid);
	},

	// 重置本地ID【通常情况不使用此函数】
	resetLocalID() {
		this._cleanLocalID();
		this._loadLocalID();
	},

	// avatar说话内容
	talkMsgs(){
		let res = null;
		if(this.maincfg && this.maincfg.info){
			let cfg = this.maincfg.info.extcfg;
			if(typeof(cfg) == 'object' && cfg.talk_data){
				res = cfg.talk_data;
			}
		}
		return res;
	},
	// 添加监听处理器
	// handler是回调对象，按实际需要可实现以下一个或多个回调：
	// .onJCInit(channelid, gameid, isoffical){}
	// .onJCAccountID(accountid, sessionid){}
	// .onJCLocalUUID(localuuid){}
	// .onJCUserInfo(userinfo){}
	// .onJCLaunchInfo(lauchscene, lauchfromid, launchparam){}
	// .onJCGameStart(startmode, extparam){}
	// .onJCGameOver(score, bsuccess, extparam){}
	addHandler(handler) {
		this._checkListener();
		if (!handler) {
			return false;
		}
		let obj = this._handlers.find(element => {
			return element == handler;
		});
		if (!obj) {
			this._handlers.push(handler);
			this._initHandler(handler);
		}
		return true;
	},

	// 删除监听处理器
	delHandler(handler) {
		if (!handler || !this._handlers) {
			return false;
		}
		let idx = this._handlers.findIndex((item) => {
			return item == handler;
		});
		if (idx < 0) {
			return true;
		}
		this._handlers.splice(idx, 1);
		return true;
	},

	// 设置sdk回调
	setResponse(rsp) {
		this.rsp = rsp;
	},

	// 设置视频广告是否开启开关【使用login.fire无需手动调用此函数】
	setADSwitch(turnoff) {
		this._offad = turnoff;
	},

	// 设置banner广告是否开启开关【使用login.fire无需手动调用此函数】
	setBannerADSwitch(turnoff) {
		this._offbanner = turnoff;
	},

	// 设置插屏广告是否开启开关【使用login.fire无需手动调用此函数】
	setInsertADSwitch(turnoff) {
		this._offinsert = turnoff;
	},

	// 设置是否开启审核开关【使用login.fire无需手动调用此函数】
	setAuditSwitch(isaudit) {
		this.isVerify = isaudit;
	},

	// 设置是否是测试模式开关【使用login.fire无需手动调用此函数】
	setTestModeSwitch(istestmode) {
		this._istestmode = istestmode;
	},

	// 设置每日视频播放限制数量（-1为不限制）【使用login.fire无需手动调用此函数】
	setVideoLimit(cnt) {
		this._dayvideocount = -1;
		if (typeof (cnt) == 'number') {
			this._dayvideocount = cnt;
		}
	},

	// 设置存储是否开启云存储开关
	setStorageFlag(iscloud) {
		this.ccstorage.setCloudFlag(iscloud);
	},

	// 获取是否开启云存储标识
	getStorageFlag() {
		return this.ccstorage.getCloudFlag();
	},

	// 获取服务器当前时间【登陆完成后才能调用】
	getServerNowTime() {
		// let tm = new Date().getTime();
		// return this.loginSrvTime + (tm - this.loginLocalTime) / 1000;
		return this.loginSrvTime;
	},

	// 获取AccountID
	getUserID() {
		return this.accountID;
	},

	// 获取是否是节假日
	isHoliday(){
		return this._isholiday;
	},

	// 获取健康游戏时间
	getGreenTime(){
		return this._greentime;
	},

	isTestMode(){
		return this._istestmode;
	},

	//游戏相关-------------------------------------------------------
	// 游戏开始
	// startmode: 0:正常开始游戏 1:重新开始 2:再来一次
	gameStart(startmode, extparam) {
		if (!this._inited) {
			return;
		}
		switch (startmode) {
			case 0:
				this.gamelog.logStartGame(extparam);
				break;
			case 1:
				this.gamelog.logRestartGame(extparam);
				break;
			case 2:
				this.gamelog.logTryAgain(extparam);
				break;
		}
		this._handleGameStart(startmode, extparam);
	},

	// 游戏结束
	gameOver(score, bsuccess, extparam) {
		if (!this._inited) {
			return;
		}
		this.gamelog.logGameover(extparam, score, bsuccess ? 1 : 0);
		this._handleGameOver(score, bsuccess, extparam);
	},

	// 游戏显示【无需手动调用】
	gameShow(showinfo) {
		if (!this._inited) {
			return;
		}
		this.gamelog.logShow();
		this._handleGameShow(showinfo);
	},

	// 游戏隐藏【无需手动调用】
	gameHide() {
		if (!this._inited) {
			return;
		}
		this.gamelog.logHide();
		this._handGameHide();
	},

	// 使用道具
	gameUseItem(itemid, itemcount, reson, extparam) {
		if (!this._inited) {
			return;
		}
		this.gamelog.logUseItem(itemid, itemcount, reson, extparam);
	},

	// 获得道具
	gameGetItem(itemid, itemcount, reson, extparam) {
		if (!this._inited) {
			return;
		}
		this.gamelog.logProductItem(itemid, itemcount, reson, extparam);
	},

	gameVibrate() {
		return this.plat && this.plat.PTVibrateShort && this.plat.PTVibrateShort();
	},

	gameVibrateLong() {
		return this.plat && this.plat.PTVibrateLong && this.plat.PTVibrateLong();
	},

	// 开始录屏
	gameStartRecord(duration, cb) {
		return this.plat && this.plat.PTStartRecord && this.plat.PTStartRecord(duration, cb);
	},

	// 停止录屏
	gameStopRecord(cb) {
		return this.plat && this.plat.PTStopRecord && this.plat.PTStopRecord(cb);
	},

	// 是否在录屏
	gameIsRecording() {
		return this.plat && this.plat.PTIsRecording && this.plat.PTIsRecording();
	},
	//-----------------------------------------------------------------

	// 以下函数无需手动调用

	setAccountID(accountid, sessionid, exobj) {
		this.accountID = accountid;
		this.sessionID = sessionid;
		this.accountobj = exobj;
		if (exobj && exobj.wid) {
			this.weakID = exobj.wid;
		}
		if (exobj && exobj.sid) {
			this.strongID = exobj.sid;
		}
		if (exobj && exobj.ptid) {
			this.ptID = exobj.ptID;
		}
		this.share.setAccountID(accountid, sessionid, exobj);
		this.gamelog.setAccountID(accountid, sessionid, exobj);
		this.login.setAccountID(accountid, sessionid, exobj);
		this.cloud.setAccountID(accountid, sessionid, exobj);
		this.ad.setAccountID(accountid, sessionid, exobj);
		this.stat.setAccountID(accountid, sessionid, exobj);
		this.notify.setAccountID(accountid, sessionid, exobj);
		this.voodoo.setAccountID(accountid, sessionid, exobj);
		this.mail.setAccountID(accountid, sessionid, exobj);
		this.recharge.setAccountID(accountid, sessionid, exobj);

		this._handleAccountID(accountid, sessionid, exobj);
		// this.logined = this.accountID && this.accountID != '';
	},

	setLocalUUID(uuid) {
		this.localUUID = uuid;
		this.gamelog.setLocalUUID(uuid);
		this.cloud.setLocalUUID(uuid);
		this.login.setLocalUUID(uuid);
		this.share.setLocalUUID(uuid);
		this.ad.setLocalUUID(uuid);
		this.stat.setLocalUUID(uuid);
		this.notify.setLocalUUID(uuid);
		this.voodoo.setLocalUUID(uuid);
		this.mail.setLocalUUID(uuid);
		this.recharge.setLocalUUID(uuid);

		this._handleLocalID(uuid);
	},

	setNickName(nickname) {
		if (nickname && nickname != '' && this.nickName != nickname) {
			this.nickName = nickname;
			this.gamelog.setNickName(nickname);
			this.cloud.setNickName(nickname);
			this.login.setNickName(nickname);
			this.share.setNickName(nickname);
			this.ad.setNickName(nickname);
			this.stat.setNickName(nickname);
			this.notify.setNickName(nickname);
			this.voodoo.setNickName(nickname);
			this.mail.setNickName(nickname);
			this.recharge.setNickName(nickname);
			this._handleNickName(nickname);
		}
		this.rank.setNickName(nickname);
	},

	setFromAppID(appid, scene, paramobj) {
		this.lastscene = scene;
		if (appid) {
			this.fromID = appid;
			this.gamelog.setFromAppID(appid);
			this.cloud.setFromAppID(appid);
			this.login.setFromAppID(appid);
			this.share.setFromAppID(appid);
			this.ad.setFromAppID(appid);
			this.stat.setFromAppID(appid);
			this.notify.setFromAppID(appid);
			this.voodoo.setFromAppID(appid);
			this.voodoo.setLaunchInfo(scene, appid, paramobj);
			this.mail.setFromAppID(appid);
			this.recharge.setFromAppID(appid);

		}
		if (paramobj) {
			this.lauchparam = paramobj;
		}
		this._handleLauchInfo(scene, appid, paramobj);
	},

	setSystemInfo(info) {
		this.systemInfo = info;
		this.gamelog.setSystemInfo(info);
		this.cloud.setSystemInfo(info);
		this.login.setSystemInfo(info);
		this.share.setSystemInfo(info);
		this.ad.setSystemInfo(info);
		this.stat.setSystemInfo(info);
		this.notify.setSystemInfo(info);
		this.voodoo.setSystemInfo(info);
		this.mail.setSystemInfo(info);
	},

	setUserInfo(info) {
		if (!this._userinfo) {
			this._userinfo = {};
		}
		this.login.appendUserInfo(this._userinfo, info);
		// this.userInfo = info;
		if (this._userinfo.nickName) {
			this.setNickName(this._userinfo.nickName);
		} else if (this._userinfo.nickname) {
			this.setNickName(this._userinfo.nickname);
		}
		this.share.setUserInfo(this._userinfo);
		this.cloud.setUserInfo(this._userinfo);
		this.rank.setUserInfo(this._userinfo);
		this._loadShareConfig(info);
		this._handleUserInfo(this._userinfo);
	},

	setServerTime(srvtime) {
		if (!srvtime) {
			return;
		}
		this.loginSrvTime = srvtime;
		if(this._countid){
			clearInterval(this._countid);
			this._countid = 0;
		}
		this._countid = setInterval(this._countservertime.bind(this), 1000);
		this.loginLocalTime = new Date().getTime();
		this.notify.setServerTime(srvtime, this.loginLocalTime);
		this._handleServerTime(srvtime, this.loginLocalTime);
	},

	_countservertime(){
		this.loginSrvTime++;
	},

	_loadLocalID() {
		let id = storage.get(_GAME_LOCALUUID);
		let bneedsave = false;
		if (!id) {
			id = this.makeUUID();
			bneedsave = true;
		}
		this.setLocalUUID(id);
		if (bneedsave) {
			this._saveLocalID();
		}
	},

	_saveLocalID() {
		if (!storage.set(_GAME_LOCALUUID, this.localUUID)) {
			setTimeout(this._saveLocalID.bind(this), 5000);
		}
	},

	_cleanLocalID() {
		storage.remove(_GAME_LOCALUUID);
	},

	_loadLaunchInfo() {
		if (this.plat && this.plat.getLaunchInfo) {
			this.lauchInfo = this.plat.getLaunchInfo();
		} else {
			this.lauchInfo = null;
		}

		this.gamelog.handlelaunch(this.lauchInfo, (sharetype, shareparam, shareuuid, inviterid,
			fromid, fromscene, launchobj) => {
			this.shareInfo.type = sharetype;
			this.shareInfo.param = shareparam;
			this.shareInfo.id = shareuuid;
			this.shareInfo.inviter = inviterid;
			console.log('[shareinfo]');
			console.log(JSON.stringify(this.shareInfo));
			this.setFromAppID(fromid, fromscene, launchobj);
		});
	},

	_loadSystemInfo() {
		if (this.plat && this.plat.getSystemInfo) {
			let info = this.plat.getSystemInfo();
			this.gamelog.logLaunchSystemInfo(info);

			this.setSystemInfo(info);
		}
	},

	_loadConfig(cfgsuccesscb, cfgfailcb) {
		this.initGlobalConfig(cfgsuccesscb, cfgfailcb);
	},

	_loadShareConfig(info) {
		if (info) {
			this.initShareConfig(null, null, true);
		} else if (!this.sharecfg.info) {
			this.initShareConfig();
		}
	},

	_initHandler(obj) {
		if (this.gameID && obj.onJCInit) {
			obj.onJCInit(this.channelID, this.gameID, this.isOffical);
		}
		if (this.accountID && this.sessionID && obj.onJCAccountID) {
			obj.onJCAccountID(this.accountID, this.sessionID, this.accountobj);
		}
		if (this.localUUID && obj.onJCLocalUUID) {
			obj.onJCLocalUUID(this.localUUID);
		}
		if (this.nickName && obj.onJCNickName) {
			obj.onJCNickName(this.nickName);
		}
		if (this._userinfo && obj.onJCUserInfo) {
			obj.onJCUserInfo(this._userinfo);
		}
		if (this.fromID && obj.onJCLaunchInfo) {
			obj.onJCLaunchInfo(this.scene, this.fromID, this.lauchparam);
		}
	},

	_handleInit(channelid, gameid, isoffical) {
		this._handlers.forEach(element => {
			if (element.onJCInit) {
				element.onJCInit(channelid, gameid, isoffical);
			}
		});
	},

	_handleAccountID(accountid, sessionid, exobj) {
		this._handlers.forEach(element => {
			if (element.onJCAccountID) {
				element.onJCAccountID(accountid, sessionid, exobj);
			}
		});
	},

	_handleLocalID(localuuid) {
		this._handlers.forEach(element => {
			if (element.onJCLocalUUID) {
				element.onJCLocalUUID(localuuid);
			}
		});
	},

	_handleNickName(nickname) {
		this._handlers.forEach(element => {
			if (element.onJCNickName) {
				element.onJCNickName(nickname);
			}
		});
	},

	_handleUserInfo(userinfo) {
		this._handlers.forEach(element => {
			if (element.onJCUserInfo) {
				element.onJCUserInfo(userinfo);
			}
		});
	},

	_handleLauchInfo(lauchscene, lauchfromid, launchparam) {
		this._handlers.forEach(element => {
			if (element.onJCLaunchInfo) {
				element.onJCLaunchInfo(lauchscene, lauchfromid, launchparam);
			}
		});
	},

	_handleServerTime(servertime, localtime) {
		this._handlers.forEach(element => {
			if (element.onJCServerTime) {
				element.onJCServerTime(servertime, localtime);
			}
		});
	},

	_handleGameStart(startmode, extparam) {
		this._handlers.forEach(element => {
			if (element.onJCGameStart) {
				element.onJCGameStart(startmode, extparam);
			}
		});
	},

	_handleGameOver(score, bsuccess, extparam) {
		this._handlers.forEach(element => {
			if (element.onJCGameOver) {
				element.onJCGameOver(score, bsuccess, extparam);
			}
		});
	},

	_handleGameShow(showinfo) {
		if (showinfo && showinfo.scene) {
			this.lastscene = showinfo.scene;
		}
		this._handlers.forEach(element => {
			if (element.onJCGameShow) {
				element.onJCGameShow(showinfo);
			}
		});
		this.event.emit(this.event.Events.APP_SHOW, showinfo);
	},

	_handGameHide(hideinfo) {
		this._handlers.forEach(element => {
			if (element.onJCGameHide) {
				element.onJCGameHide(hideinfo);
			}
		});
		this.event.emit(this.event.Events.APP_HIDE);
	},

	_handleNotifyMsgs(lst, notifyobj) {
		this._handlers.forEach(element => {
			if (element.onJCNotifyMsg) {
				element.onJCNotifyMsg(lst, notifyobj);
			}
		});
	},

	_checkListener() {
		if (!this._listenerID) {
			this._listenerID = notification.on(_GAME_INFO_HANDLER_BINDING, (msgid, handler) => {
				this.addHandler(handler);
			}, this);
		}
		if (!this._unlistenerID) {
			this._unlistenerID = notification.on(_GAME_INFO_HANDLER_UNBINDING, (msgid, handler) => {
				this.delHandler(handler);
			}, this);
		}
	},

	_fixChannelID(id, polyid) {
		this.channelID = parseInt(id);
		this.polyID = polyid? parseInt(polyID): 0;
		//TODO:: 等creator支持qq小程序后 修改该方法
		if (cc.sys.platform === cc.sys.WECHAT_GAME) {
			if (window.qq) {
				this.channelID = this.channel.QQ_MINI;
			} else if (window.tt) {
				this.channelID = this.channel.TT;
			} else {
				this.channelID = this.channel.WECHAT;
			}
		} else if (cc.sys.platform == cc.sys.QQ_PLAY) {
			this.channelID = this.channel.QQ_PLAY;
		} else if (cc.sys.platform == cc.sys.MOBILE_BROWSER || cc.sys.platform == cc.sys.DESKTOP_BROWSER) {
			if (this.channelID == this.channel.WECHAT || this.channelID == this.channel.QQ_PLAY) {
				this.channelID = this.channel.TEST;
			}
		}

		let nid = this.polyID>0? this.polyID: this.channelID;

		switch (nid) {
			case this.channel.WECHAT:
				this.plat = require('./platform/plat_wechat');
				break;
			case this.channel.QQ_PLAY:
				this.plat = require('./platform/plat_qq');
				break;
			case this.channel.OPPO:
				this.plat = require('./platform/plat_oppo');
				break;
			case this.channel.VIVO:
				this.plat = require('./platform/plat_vivo');
				break;
			case this.channel.FB:
				this.plat = require('./platform/plat_fb');
				break;
			case this.channel.BAIDU:
				this.plat = require('./platform/plat_baidu');
				break;
			case this.channel.TT:
				this.plat = require('./platform/plat_tt');
				break;
			case this.channel.QQ_MINI:
				this.plat = require('./platform/plat_qqmini');
				break;
			case this.channel.QUICK:
				this.plat = require('./platform/plat_quick');
				break;
			case this.channel.ANDROID:
				this.plat = require('./platform/plat_android');
				break;
			case this.channel.GOOGLE:
				this.plat = require('./platform/plat_google');
				break;
			default:
				this.plat = require('./platform/plat_h5');
				break;
		}

		if (this.plat) {
			this.plat.init(this, nid);
		}
	},

	_handleCfg(cfg) {
		if (!cfg) {
			return;
		}
		cc.gg.basecfg = cfg;
		this.version = cfg._serverversion ? cfg._serverversion : '';
		this.config_version = cfg.config_version ? cfg.config_version : '';

		this.maincfg.info = cfg;
		this.maincfg.info.openAD ? this.setADSwitch(false) : this.setADSwitch(true);
		this.maincfg.info.openAdBanner ? this.setBannerADSwitch(false) : this.setBannerADSwitch(true);
		this.maincfg.info.openADInsert ? this.setInsertADSwitch(false) : this.setInsertADSwitch(true);
		this.maincfg.info.isTestMode ? this.setTestModeSwitch(true) : this.setTestModeSwitch(false);
		this.maincfg.info.blackTech ? this.setAuditSwitch(false) : this.setAuditSwitch(true);
		this.setVideoLimit(this.maincfg.info.video_count);
		this._bannerids = this.maincfg.info.wxBannerAdId ? this.maincfg.info.wxBannerAdId.split('|') : [];
		this._bannertimeout = this.maincfg.info.wxBannerOvertime ? this.maincfg.info.wxBannerOvertime : 0;
		if (this.maincfg.info.banner_cfg) {
			try {
				this.maincfg.info.bannerCfg = JSON.parse(this.maincfg.info.banner_cfg);
			} catch (err) {

			}
		}
		if (this.maincfg.info.statInvTime) {
			this.stat.setHeart(this.maincfg.info.statInvTime);
		}
		if (typeof (this.maincfg.info.share_waitsec) == 'number') {
			this.plat && (this.plat.SHARE_TIME = this.maincfg.info.share_waitsec * 1000);
		}
		if(this.maincfg.info.ext_json_cfg){
			try {
				this.maincfg.info.extcfg = JSON.parse(this.maincfg.info.ext_json_cfg);
			} catch (err) {

			}
		}
		// this.ADBanner_init();
	},

	/**
	 * 检查当日是否还可以播放视频激励广告
	 * 使用统一的每日激励广告限额来检查
	 * */
	canIShare(stype) {
		if (this._offad) {
			return true;
		} else {
			return this.adshare.checkAdShareCommon();
		}
	},

	/**
	 *
	 * @param {string} appid 要跳转的appid
	 * @param {string} apppath 传入的path参数
	 * @param {object} param_kvlst : Array，key/value参数的列表：eg：[{key: "aaa", value: "bbb"}]
	 * @param {object} cb : 跳转后的回调函数
	 * @param {string} env : 跳转小程序的版本（需要时传入，一般不传）：develop：开发版；trial：体验版
	 */
	gotoJCApp(appid, apppath, param_kvlst, cb, env) {
		let exobj = {
			accountId: this.accountID,
			sessionId: this.sessionID,
			gameId: this.gameID,
			channelId: this.channelID
		};
		if (param_kvlst && param_kvlst instanceof Array) {
			param_kvlst.forEach(element => {
				exobj[element.key] = element.value;
			});
		}
		if (!this.plat) {
			cb && cb(-1000);
		} else {
			this.plat.PTGotoApp(appid, apppath, exobj, cb, this.gameID, env);
		}
	}
};

cc.jc = JCFW;

module.exports = JCFW;
