const JCFW_CLASS = 'com/jc/jcfw/JcSDK';
const TAG = '[PLAT_GOOGLE]';
var AdObj = function (type) {
  this.type = type;
  this.onResize = function () {
  }
  this.showBanner = function () {
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'showBanner', '()V');
  }
  this.hideBanner = function () {
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'hideBanner', '()V');
  }
  this.showRewardVideo = function () {
    cc.currentVideoAd = this;
    console.log(TAG, 'showRewardVideo ', cc.currentVideoAd);
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'showRewardVideo', '()V');
  }
  this.showInterstitial = function () {
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'showInterstitial', '()V');
  }
  this.showActArr = [this.showBanner, this.showRewardVideo, this.showInterstitial];
  this.hideActArr = [this.hideBanner, this.hideBanner, this.hideBanner];
  this.show = function () {
    return new Promise((resolve, reject) => {
      this.showActArr[this.type].call(this);
      resolve && resolve();
    });
  }
  this.hide = function () {
    this.hideActArr[this.type].call(this);
  }
}
/**
 * 支付相关
 * @class
 */
var PayObj = function(aowner) {
  let self = this;
  let _owner = aowner;
  this.refreshmaps = {};

  this.productMap = new Map();
  /**
   * 刷新商品数据
   * 再次刷新可以不传参数
   * @param {string | [string]} skuList 单个商品id或者多个商品id(以半角逗号或者|隔开), 或多个商品id组成的数组
   * @param {PayObj~refreshSuccessCb} successCb    支付成功回调, 如果为空的话, 支付成功后, 会emit PAY_SUCCESS事件
   * @param {PayObj~refreshFailCb} failCb       支付失败回调, 如果为空的话, 支付失败后, 会emit PAY_FAILED事件
   */
  this.refreshProducts = function (skuList, successCb, failCb) {
    _owner.gamelog.logButtonClick('pay_google', skuList, 'ref_goods[r]');

    // self.refreshSuccessCb = successCb;
    // self.refreshFailCb = failCb;

    let skuListStr = '';
    if (!skuList && this.skuListStr) {
      skuListStr = this.skuListStr;
    } else {
      if (Array.isArray(skuList)) {
        skuListStr = skuList.join(',');
      } else {
        skuListStr = skuList.replace(/\|/g, ',');
      }
    }
    console.log(TAG, 'refreshProducts: ', skuListStr);
    if (!skuListStr) {
      failCb && failCb();
      _owner.gamelog.logButtonClick('pay_google', {err: -9527}, 'ref_goods[fail]');
      return;
    }

    if(successCb || failCb){
      self.refreshmaps[skuListStr] = {
        IDs: skuListStr.split(','),
        OKCb: successCb,
        notOKCb: failCb,
      };
    }

    this.skuListStr = skuListStr;
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'refreshProducts', '(Ljava/lang/String;)V', this.skuListStr);
  }
  /**
   * 获取产品信息
   * @param {string | [string]} skuList 单个商品id或者多个商品id(以半角逗号或者|隔开), 或多个商品id组成的数组, 为空的话, 获取所有产品列表
   * @param {PayObj~refreshSuccessCb} successCb    支付成功回调, 如果为空的话, 支付成功后, 会emit PAY_SUCCESS事件
   * @param {PayObj~refreshFailCb} failCb       支付失败回调, 如果为空的话, 支付失败后, 会emit PAY_FAILED事件
   */
  this.queryProducts = function (skuList, successCb, failCb) {
    _owner.gamelog.logButtonClick('pay_google', skuList, 'query_goods[r]');
    if (!skuList && !self.skuListStr) {
      failCb && failCb('call cc.jcpay.refreshProducts(skuList) first!!');
      _owner.gamelog.logButtonClick('pay_google', {err: -9527}, 'query_goods[fail]');
      return;
    }
    if (!skuList) {
      successCb && successCb(self.productList);
      _owner.gamelog.logButtonClick('pay_google', self.productList, 'query_goods[ok]');
      return;
    }
    let skuArr;
    if (Array.isArray(skuList)) {
      skuArr = skuList;
    } else {
      skuArr = skuList.replace(/\|/g, ',').split(',');
    }
    let notEnough = false;
    for (let sku of skuArr) {
      if (!self.productMap.has(sku)) {
        notEnough = true;
        break;
      }
    }
    if (notEnough) {
      self.refreshProducts(skuList, successCb, failCb);
    } else {
      let results = [];
      for (let sku of skuArr) {
        results.push(self.productMap.get(sku));
      }
      successCb && successCb(results);
      _owner.gamelog.logButtonClick('pay_google', results, 'query_goods[ok]');
    }
  }

  /**
   * @callback PayObj~refreshSuccessCb
   * @param {[object]} products 最新的产品信息
   */
  /**
   * @callback PayObj~refreshFailCb
   * @param {string} msg 服务返回的错误信息
   */

  /**
   * 购买
   * @param {string} productId  从productList中获取的产品ID
   * @param {PayObj~purchaseSuccessCb} successCb    支付成功回调
   * @param {PayObj~purchaseFailCb} failCb       支付失败回调
   * */
  this.purchase = function(productId, successCb, failCb) {
    console.log(TAG, 'begin purchase: ', productId);
    self.purchaseSuccessCb = successCb;
    self.purchaseFailCb = failCb;
    jsb.reflection.callStaticMethod(JCFW_CLASS, 'buyProduct', '(Ljava/lang/String;)V', productId);
    _owner.gamelog.logButtonClick('pay_google', {id: productId}, 'buy_goods[r]');
  }
  /**
   * @callback PayObj~purchaseSuccessCb
   * @param {Object} product
   */
  /**
   * @callback PayObj~purchaseFailCb
   * @param {number} data.errcode 错误代码 1: 用户取消, 2: 其他
   * @param {Object} data.product 购买的商品
   * @param {string} data.errmsg  服务返回的错误信息
   */

  /**
   * 供java调用的支付错误回调
   */
  this.payError = function (errcode, productId) {
    console.log(TAG, 'pay error with code: ', errcode, productId);
    let msg = errcode === 0 ? 'user cancel' : 'pay error';
    let product = self.productMap.get(productId);
    self.purchaseFailCb && self.purchaseFailCb({product, errmsg: msg, errcode});
    if (errcode === 1) {
      cc.jc.event.emit(cc.jc.event.Events.PAY_FAILED, {product, msg});
    } else {
      cc.jc.event.emit(cc.jc.event.Events.PAY_CANCEL, {product, msg});
    }
    let item = {
      id: productId,
      err: errcode,
    };
    _owner.gamelog.logButtonClick('pay_google', item, 'buy_goods[fail]');
  }
  /**
   * 供java调用的支付成功回调
   */
  this.paySuccess = function (productId) {
    console.log(TAG, 'pay success: ', productId);
    let product = self.productMap.get(productId);
    self.purchaseSuccessCb && self.purchaseSuccessCb(product);
    cc.jc.event.emit(cc.jc.event.Events.PAY_SUCCESS, product);
    _owner.gamelog.logButtonClick('pay_google', {id: productId}, 'buy_goods[ok]');
  }
  /**
   * 供java调用的刷新产品列表回调
   */
  this.onProductsUpdate = function (jsonStr) {
    let robj = null;
    let rkey = '';
    try {
      let datas = JSON.parse(jsonStr);
      // console.log(TAG, 'onProductsUpdate:' + datas.length, jsonStr);
      let idlst = [];
      for (let i = 0; i < datas.length; i++) {
        // console.log(TAG, '[onProductsUpdate]'+ i, datas[i]);
        self.productMap.set(datas[i].id, datas[i]);
        idlst.push(datas[i].id);
      }
      console.log(TAG, '[onProductsUpdate]', idlst);

      self.productList = [...self.productMap.values()];
      // self.refreshSuccessCb && self.refreshSuccessCb(datas);

      for(let key in self.refreshmaps){
        let obj = self.refreshmaps[key];
        if(!obj){
          continue;
        }
        let lst = obj.IDs;
        let bok = true;
        console.log(TAG, '[refreshmaps]', lst);
        for(let i = 0; i < idlst.length; i++){
          if(!lst.includes(idlst[i])){
            bok = false;
            break;
          }
        }
        if(bok){
          robj = obj;
          rkey = key;
          break;
        }
      }

      if(robj){
        console.log(TAG, 'find obj!');
        try{
          robj.OKCb && robj.OKCb(datas);
          robj.OKCb = null;
          robj.notOKCb = null;
          self.refreshmaps[rkey] = null;
        }catch(e){

        }
      }

      _owner.gamelog.logButtonClick('pay_google', idlst, 'ref_goods[ok]');
    } catch (err) {
      console.error(TAG, 'convert products json error', JSON.stringify(err));
      // self.refreshFailCb && self.refreshFailCb();
      if(robj){
        try{
          robj.notOKCb && robj.notOKCb();
          robj.notOKCb = null;
          robj.OKCb = null;
          self.refreshmaps[rkey] = null;
        }catch(e){

        }
      }
      _owner.gamelog.logButtonClick('pay_google', null, 'ref_goods[fail]');
    }

  }
}
var PLAT_GOOGLE = {
  SHARE_FAIL: 20,
  SHARE_TIME: 3000,
  shareCount: {},

  getLaunchInfo() {
    return null;
  },

  getSystemInfo() {
    return null;
  },

  /** example:
   *
   * @param {object} btninfo :
   *  {
   *      width: 100,
   *      height: 100,
   *      name: logo.png
   * }
   * @param {function} cb
   */
  getUserInfo(btninfo, cb) {

  },

  destoryAuth() {

  },

  setUserInfo(dstinfo, ptinfo) {
    dstinfo.nickname = ptinfo.nickName;
    dstinfo.country = ptinfo.country;
    dstinfo.province = ptinfo.province;
    dstinfo.city = ptinfo.city;
    dstinfo.avatar_url = ptinfo.avatarUrl;
    dstinfo.sex = ptinfo.gender;
  },

  setTokenInfo(dstinfo, tokeninfo) {
    if (tokeninfo && tokeninfo.rawData) {
      dstinfo.rawData = tokeninfo.rawData;
      dstinfo.signature = tokeninfo.signature;
      dstinfo.encryptedData = tokeninfo.encryptedData;
      dstinfo.iv = tokeninfo.iv;
    }
  },

  init(owner, channelid) {
    this._owner = owner;
    this._ptid = channelid;
    let self = this;
    cc.videoClose = function (isEnd) {
      console.log(TAG, 'video ad close, isEnd:', isEnd, cc.currentVideoAd);
      if (cc.currentVideoAd) {
        cc.currentVideoAd.onClose && cc.currentVideoAd.onClose(isEnd);
        cc.currentVideoAd = null;
      }
    }
    cc.loginSuccess = function(dataStr) {
      console.log(TAG, 'google login success:', dataStr);
      self.loginSuccess(dataStr);
    }
    cc.loginFailed = function (dataStr) {
      console.log(TAG, 'google login failed with: ', dataStr);
      self.loginFailed(dataStr);
    }
    this.initPay();
  },
  bindController(ctrl) {
    this._man = ctrl;
    let nd = cc.find('Canvas');
    if (nd) {
      let cv = nd.getComponent(cc.Canvas);
      if (cv) {
        this._dr = cv.designResolution;
      }
    }
  },

  commonShare(aimg, atitle, stype, param, successcb, failcb, exinfo) {

  },

  captureShare(rc, title, stype, param, successcb, failcb, exinfo) {

  },


  PTInitShare(normalinfo) {


  },

  PTShare(content, imgurl, param, successcb, failcb) {

  },

  PTShareCapture(content, rc, param, successcb, failcb) {

  },

  PTOpenKF(cb, info) {
    cb && cb(false);
  },

  /**
   * @param {string} str 显示的文字
   * @param {icon} icon 图标类型 这里不作处理
   * */
  PTShowToast(str, icon) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID && str) {
      console.log(TAG, 'PTShowToast', str);
      jsb.reflection.callStaticMethod(JCFW_CLASS, 'showToast', '(Ljava/lang/String;)V', str);
    }
  },

  PTGotoApp(appid, apppath, exobj, cb, qdid, env, acontent) {
    cb && cb(-1000, null);
  },

  PTLogin(info, successcb, failcb) {
    var self = this;
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      var uid = jsb.reflection.callStaticMethod(JCFW_CLASS, 'generateUid', '()Ljava/lang/String;');
      console.log(TAG, 'success generate uid: ', uid);
      self._owner.setNickName(uid.length>7 ? uid.substring(0, 7) : uid);
      self._owner.login.login(uid, successcb, failcb, uid);
      // 暂时移除google登陆功能
      // jsb.reflection.callStaticMethod(JCFW_CLASS, 'signToGoogle', '()V');
      // self.loginSuccess = function (res) {
      //   try {
      //     let data = JSON.parse(res);
      //     self._owner.setNickName(data.nickname);
      //     self._owner.setAvatar(data.avatar);
      //     self._owner.login.login(data.token, successcb, failcb, data.uid);
      //   } catch (err) {
      //     failcb && failcb(0, -1000, 'parse json error!');
      //   }
      // }
      // self.loginFailed = function (res) {
      //   try {
      //     console.log(TAG, 'fail login to google, use customer uid', res);
      //     self._owner.setNickName(uid.length>7 ? uid.substring(0, 7) : uid);
      //     self._owner.login.login(uid, successcb, failcb, uid);
      //   } catch (err) {
      //     failcb && failcb(0, -1000, 'parse json error!');
      //   }
      // }
    } else {
      failcb && failcb(0, -1000, 'not android platform!');
    }
  },


  //广告相关
  PTInitADService() {

  },


  PTCreateBannerAD(adid, st) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      return new AdObj(0);
    } else {
      return null;
    }
  },

  PTCreateVideoAD(adid) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      return new AdObj(1);
    } else {
      return null;
    }
  },

  PTCreateInsertAD(adid) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      return new AdObj(2);
    } else {
      return null;
    }
  },

  PTSetVideoADCallback(adobj, adowner, cb) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      adobj.onClose  = function (res) {
        console.log(TAG, '[ad]onClose:' + JSON.stringify(res));
        // 用户点击了【关闭广告】按钮
        if (res) {
          // 正常播放结束，可以下发游戏奖励
          cb && cb(true, adowner, res);
        } else {
          // 播放中途退出，不下发游戏奖励
          cb && cb(false, adowner, res);
        }
        // ad.offClose && ad.offClose(closeCb);
      };

    }
  },

  PTVibrateShort(successcb, failcb) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      jsb.reflection.callStaticMethod(JCFW_CLASS, 'vibrate', '(I)V', 0);
      successcb && successcb();
    } else {
      failcb && failcb();
    }
  },

  PTVibrateLong(successcb, failcb) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      jsb.reflection.callStaticMethod(JCFW_CLASS, 'vibrate', '(I)V', 1);
      successcb && successcb();
    } else {
      failcb && failcb();
    }
  },

  PTOpenDataContext() {
    return null;
  },

  PTADWatch(adid, successcb, failcb) {
    if (cc.sys.isNative && cc.sys.os === cc.sys.OS_ANDROID) {
      let videoAd = this.PTCreateInsertAD(adid);
      videoAd.onClose = function (res) {
        console.log(TAG, '[ad]onClose:' + JSON.stringify(res));
        // 用户点击了【关闭广告】按钮
        if (res) {
          // 正常播放结束，可以下发游戏奖励
          successcb&&successcb(adid, res, adid);
        } else {
          // 播放中途退出，不下发游戏奖励
          failcb && failcb(adid, res, true, adid);
        }
        // ad.offClose && ad.offClose(closeCb);
      };
      videoAd.show();

    } else {
      failcb && failcb(adid, -1000, true, adid);
    }
  },
  initPay() {
    this.pay = new PayObj(this._owner);
    cc.jcpay = this.pay;
    cc.jc.pay = this.pay;
  }
};

module.exports = PLAT_GOOGLE;
