var httpclient = require('../common/httpclient');

let jcrank = {
  init(channelid, gameid, isoffical, owner, url){
    this.owner = owner;
    this.gameid = gameid;
    this.channelid = channelid;
    this.isoffical = isoffical;
    this.url = url.replace('webapp/index.php', 'api/games');
    console.log("[jcrankctrl]init:"+gameid + "|" + channelid + "|" + isoffical);
  },
  setNickName(nickname) {
    if(nickname){
      this.nickname = nickname;
    }else{
      this.nickname = '匿名玩家';
    }

  },
  setAvatar(avatar) {
    if(avatar){
      this.avatar_url = avatar;
    }else{
      this.avatar_url = 'https://resource.kingsome.cn/share/1004/normal/5d42b6fb8e19260c1a16eefa.png'
    }
  },
  setUserInfo(info){
    if (info) {
      info.avatar_url && this.setAvatar(info.avatar_url);
      // this.setNickName(info.nickname);
    }
  },
  canShowSelf() {
    return (this.nickname || this.avatar_url);
  },
  /**
   * 世界排行榜
   * successcb各参数，
   *  - param0 用户列表
   *  - param1 当前用户的rank，显示时须+1
   *  - param2 当前用户最高分
   *  - param3 当前用户的段位
   * */
  reqRankList(successcb, failcb){
    if(!this.url){
      failcb && failcb(0, -1234, 'not inited!');
      return;
    }
    const url = `${this.url}/rank`
    let self = this;
    let data = {gameId: this.gameid, channelId: this.channelid, accountId: this.owner.accountID};
    httpclient.httpPost(url, JSON.stringify(data), (restext) => {
    // httpclient.httpPost(url, this.jsonToKeyVal(data), (restext) => {
      let obj
      try {
        obj = httpclient.JSON_parse(restext);
      } catch (err) {}
      if(obj && obj.errcode === 0){
        self.recordList = obj.records;
        if(self.recordList){
          successcb && successcb(self.recordList, obj.userRank, obj.userScore, obj.userTitle);
        }else{
          failcb && failcb(0, -1, "ranklist is not obj");
        }
      }else{
        console.log('[reqRankList]failed!'+ obj ? obj.errcode : '' +":"+ obj ? obj.errmsg : '');
        failcb && failcb(0, obj ? obj.errcode : -1235, obj ? obj.errmsg : '');
      }

    }, (errcode, errmsg) => {
      console.log('[reqRankList]failed!'+errcode+":"+errmsg);
      failcb && failcb(errcode, 0, errmsg);
    });
  },

  /**
   * 上报用户分数
   * @param {number} score 分数
   * @param {function} successcb
   * @param {function} failcb
   * @title {string}  段位
   * @extInfo {string} 额外信息
   * */
  updateRank(score, successcb, failcb, title, extinfo) {
    if(!this.url){
      failcb && failcb(0, -1234, 'not inited!');
      return;
    }
    this.nickname = this.nickname || cc.jc.nickName;
    if(!this.nickname && !this.avatar_url) {
      failcb && failcb(0, -1234, 'not login!');
      return;
    }
    const url = `${this.url}/rank/update`
    let data = {gameId: this.gameid,
      channelId: this.channelid,
      accountId: this.owner.accountID,
      score: score,
      nickname: this.nickname,
      avatar: this.avatar_url,
      needType: 1,
    };
    title && (data['rankTitle']=title);
    extinfo && (data['extInfo']=extinfo);
    httpclient.httpPost(url, JSON.stringify(data), (restext) => {
    // httpclient.httpPost(url, this.jsonToKeyVal(data), (restext) => {
      let obj
      try {
        obj = httpclient.JSON_parse(restext);
      } catch (err) {}
      if(obj && obj.errcode === 0){
        let datas = [];
        if (obj.preRecord) {
          obj.preRecord.rank += 1;
          datas.push(obj.preRecord);
        }
        datas.push({
          score: obj.userScore,
          rank: obj.userRank + 1,
          avatar: this.avatar_url,
          nickname: this.nickname,
        })
        if (obj.nextRecord) {
          obj.nextRecord.rank += 1;
          datas.push(obj.nextRecord);
        }
        successcb && successcb(datas);
      }else{
        console.log('[reqRankList]failed!'+ obj ? obj.errcode : '' +":"+ obj ? obj.errmsg : '');
        failcb && failcb(0, obj ? obj.errcode : -1235, obj ? obj.errmsg : '');
      }

    }, (errcode, errmsg) => {
      console.log('[updateRank]failed!'+errcode+":"+errmsg);
      failcb && failcb(errcode, 0, errmsg);
    });
  },
  /*
  * 挑战列表
  * */
  rankChallenge(score, successcb, failcb, count) {
    if(!this.url){
      failcb && failcb(0, -1234, 'not inited!');
      return;
    }
    count = count || 10;
    const url = `${this.url}/rank/challenge/0/${this.owner.accountID}/${count}`
    let self = this;
    httpclient.httpGet(url, (restext) => {
      let obj
      try {
        obj = httpclient.JSON_parse(restext);
      } catch (err) {}
      if(obj && obj.errcode === 0){
        successcb && successcb(obj.users);
      }else{
        console.log('[reqChallengeList]failed!'+ obj ? obj.errcode : '' +":"+ obj ? obj.errmsg : '');
        failcb && failcb(0, obj ? obj.errcode : -1235, obj ? obj.errmsg : '');
      }

    }, (errcode, errmsg) => {
      console.log('[reqChallengeList]failed!'+errcode+":"+errmsg);
      failcb && failcb(errcode, 0, errmsg);
    });
  },
  jsonToKeyVal(data) {
    let str = '';
    for (var key in data){
      if (!data.hasOwnProperty(key)){
        continue;
      }
      str += `${encodeURIComponent(key)}=${encodeURIComponent(data[key])}&`;
    }
    return str.replace(/\$$/, '');
  }
};

module.exports = jcrank;
