
var httpclient = require('../common/httpclient');
var urlbuilder = require('../common/urlbuilder');

let jcshare = {
    // LIFE-CYCLE CALLBACKS:

    // onLoad () {},

    // start () {},

    // update (dt) {},

    init(channelid, gameid, isoffical, owner, url){
        this.owner = owner;
        this.gameid = gameid;
        this.channelid = channelid;
        this.urlbd = new urlbuilder(url);
        console.log("[jcshare]init:"+gameid + "|" + channelid + "|" + isoffical);
    },

    setAccountID(accountid, sessionid){
        this.accountid = accountid;
        this.sessionid = sessionid;
    },

    setNickName(nickname){
        this.nickname = nickname;
    },

    setFromAppID(appid){
        this.fromid = appid;
    },

    setLocalUUID(uuid){
        this.localid = uuid;
    },

    setSystemInfo(info){

    },

    setUserInfo(info){
        this.sex = info.sex;
        this.avatar_url = info.avatar_url;
    },

    /**
     * 接受邀请（分级时用，大部分时候不用此接口）
     * @param {string} actionid  业务号
     * @param {string} inviterid  邀请方账号id
     * @param {object} successcb  成功回调（以下该名称都是此功能）
     * @param {object} failcb 失败回调（以下该名称都是此功能）
     */
    acceptInvite(actionid, inviterid, successcb, failcb){
        if(!actionid){
            return;
        }
        let sid = inviterid? inviterid: "";
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Fission')
        .addKV('a', 'accept')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('business_no', actionid)
        .addKV('inviter_id', sid)
        .addKV('nickname', this.nickname)
        .addKV('sex', this.sex)
        .addKV('avatar_url', this.avatar_url)
        ;

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0 || obj.errcode == 1 || obj.errcode == 2){
                console.log('[acceptInvite]success!'+JSON.stringify(obj));
                successcb && successcb();
            }else{
                console.log('[acceptInvite]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[acceptInvite]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 查询子节点数据（分级时用，大部分时候不用此接口）
     * @param {string} actionid 业务号
     * @param {object} successcb
     * @param {object} failcb
     */
    queryShareStat(actionid, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Fission')
        .addKV('a', 'getChild')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('business_no', actionid);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryShareStat]success!'+JSON.stringify(obj));
                successcb && successcb(obj.nodes);
            }else{
                console.log('[queryShareStat]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryShareStat]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 查询接受邀请人的详细信息（分级时用，大部分时候不用此接口）
     * @param {string} actionid 业务号
     * @param {object} successcb
     * @param {object} failcb
     */
    queryShareDetail(actionid, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Fission')
        .addKV('a', 'getFriendsDetail')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('business_no', actionid);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryShareDetail]success!'+JSON.stringify(obj));
                successcb && successcb(obj.nodes);
            }else{
                console.log('[queryShareDetail]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryShareDetail]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 每日任务的接受邀请【使用jcfw的话无需主动调用】
     * @param {string} actionid 任务id
     * @param {string} inviterid 邀请方账号id
     * @param {object} successcb
     * @param {object} failcb
     */
    acceptDailyInvite(actionid, inviterid, successcb, failcb){
        let nact = 0;
        let data = '';
        if(typeof(actionid) == 'number'){
            nact = actionid;
        }else if(typeof(actionid) == 'string'){
            data = actionid;
            let obj = httpclient.JSON_parse(actionid);
            if(obj){
                nact = obj.actionid? obj.actionid: 0;
            }
        }else if(typeof(actionid) == 'object'){
            let obj = actionid;
            nact = obj.actionid? obj.actionid: 0;
        }
        let sid = inviterid? inviterid: "";
        this.urlbd.clear();
        this.urlbd.addKV('c', 'DailyMission')
        .addKV('a', 'acceptInvite')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('inviter_id', sid)
        .addKV('activity_param', nact)
        .addKV('custom_data', data)
        .addKV('nickname', this.nickname)
        .addKV('sex', this.sex)
        .addKV('avatar_url', this.avatar_url)
        ;

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && (obj.errcode == 0 || obj.errcode == 1)){
                console.log('[acceptDailyInvite]success!'+JSON.stringify(obj));
                successcb && successcb();
            }else{
                console.log('[acceptDailyInvite]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[acceptDailyInvite]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 每日任务的查询接受邀请的人数
     * @param {string} actionid 业务号
     * @param {object} successcb
     * @param {object} failcb
     */
    queryDailyShareStat(actionid, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'DailyMission')
        .addKV('a', 'getInviteeNum')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('activity_param', actionid);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryDailyShareStat]success!'+JSON.stringify(obj));
                successcb && successcb(obj);
            }else{
                console.log('[queryDailyShareStat]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryDailyShareStat]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 每日任务的查询接受邀请人的详细信息
     * @param {string} actionid 业务号
     * @param {object} successcb
     * @param {object} failcb
     */
    queryDailyShareDetail(actionid, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'DailyMission')
        .addKV('a', 'getInviteeList')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('activity_param', actionid);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryDailyShareDetail]success!'+JSON.stringify(obj));
                successcb && successcb(obj.invitee_list);
            }else{
                console.log('[queryDailyShareDetail]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryDailyShareDetail]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 上报游戏分数，获取兑换码
     * @param {string} score 分数
     * @param {object} successcb
     * @param {object} failcb
    */
    queryActionCode(score, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Exchange')
        .addKV('a', 'reportScore')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('score', score);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryActionCode]success!'+JSON.stringify(obj));
                successcb && successcb(obj.exchange_code);
            }else{
                console.log('[queryActionCode]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryActionCode]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 接力活动的接受邀请
     * @param {string} actionid 任务id
     * @param {string} inviterid 邀请方账号id
     * @param {object} successcb
     * @param {object} failcb
     */
	acceptRelayInvite(actionid, inviterid, successcb, failcb){
        let sid = inviterid? inviterid: "";
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Relay')
        .addKV('a', 'acceptInvite')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('inviter_id', sid)
        .addKV('activity_param', actionid)
        .addKV('nickname', this.nickname)
        .addKV('sex', this.sex)
        .addKV('avatar_url', this.avatar_url)
        ;

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && (obj.errcode == 0 || obj.errcode == 1)){
                console.log('[acceptRelayInvite]success!'+JSON.stringify(obj));
                successcb && successcb();
            }else{
                console.log('[acceptRelayInvite]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[acceptRelayInvite]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
	},

    /**
     * 接力活动的查询接受邀请的人数
     * @param {string} actionid 业务号
     * @param {object} successcb
     * @param {object} failcb
     */
	queryRelayShareStat(actionid, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'Relay')
        .addKV('a', 'getInviteeNum')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('activity_param', actionid);

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryRelayShareStat]success!'+JSON.stringify(obj));
                successcb && successcb(obj);
            }else{
                console.log('[queryRelayShareStat]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryRelayShareStat]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 成就的接受邀请
     * @param {Array} relationidlst 有关联的成就id列表，格式如下：[[1001,3], [1002,3], [1003,3]]
     * @param {string} inviterid 邀请方账号id
     * @param {object} successcb
     * @param {object} failcb
     */
	acceptAchivementInvite(relationidlst, inviterid, successcb, failcb){
        let sid = inviterid? inviterid: "";
        this.urlbd.clear();
        this.urlbd.addKV('c', 'AchievementShare')
        .addKV('a', 'acceptInvite')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('inviter_id', sid)
        .addKV('relate_achivements', JSON.stringify(relationidlst))
        .addKV('nickname', this.nickname)
        .addKV('sex', this.sex)
        .addKV('avatar_url', this.avatar_url)
        ;

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && (obj.errcode == 0 || obj.errcode == 1)){
                console.log('[acceptAchivementInvite]success!'+JSON.stringify(obj));
                successcb && successcb();
            }else{
                console.log('[acceptAchivementInvite]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[acceptAchivementInvite]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 成就的查询接受邀请的人数
     * @param {Array} actionidlst  成就id列表，格式如下：[1001,1002,1003]
     * @param {object} successcb
     * @param {object} failcb
     */
	queryAchievementShareStat(actionidlst, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'AchievementShare')
        .addKV('a', 'getInviteeNum')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('achievement_ids', actionidlst.join());

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryAchievementShareStat]success!'+JSON.stringify(obj));
                successcb && successcb(obj);
            }else{
                console.log('[queryAchievementShareStat]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryAchievementShareStat]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    /**
     * 成就的查询接受邀请的详细信息
     * @param {Array} actionidlst  成就id列表，格式如下：[1001,1002,1003]
     * @param {object} successcb
     * @param {object} failcb
     */
	queryAchievementShareDetail(actionidlst, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'AchievementShare')
        .addKV('a', 'getInviteeList')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('achievement_ids', actionidlst.join());

        httpclient.httpGet(this.urlbd.baseurl, function(restext){
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[queryAchievementShareDetail]success!'+JSON.stringify(obj));
                successcb && successcb(obj.invitee_list);
            }else{
                console.log('[queryAchievementShareDetail]failed!'+obj.errcode+":"+obj.errmsg);
                failcb && failcb(0, obj.errcode, obj.errmsg);
            }
        }, function(errcode, errmsg){
            console.log('[queryAchievementShareDetail]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    writeShareData(key, data, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'CustomData')
        .addKV('a', 'writeData')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('key', key);

        let value = typeof(data) == 'object'? JSON.stringify(data): data;

        httpclient.httpPost(this.urlbd.baseurl, value, (restext)=>{
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[writeShareData]success!'+JSON.stringify(obj));
                successcb && successcb();
            }else{
                console.log('[writeShareData]failed!'+obj.errcode? obj.errcode: 0+":"+obj.errmsg? obj.errmsg: '');
                failcb && failcb(0, obj.errcode? obj.errcode: 0, obj.errmsg? obj.errmsg: '');
            }
        }, function(errcode, errmsg){
            console.log('[writeShareData]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    readShareData(key, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'CustomData')
        .addKV('a', 'readData')
        .addKV('session_id', this.sessionid)
        .addKV('account_id', this.accountid)
        .addKV('key', key);

        httpclient.httpGet(this.urlbd.baseurl, (restext)=>{
            var obj = httpclient.JSON_parse(restext);
            if(obj && obj.errcode == 0){
                console.log('[readShareData]success!'+JSON.stringify(obj));
                successcb && successcb(obj.data);
            }else{
                console.log('[readShareData]failed!'+obj.errcode? obj.errcode: 0+":"+obj.errmsg? obj.errmsg: '');
                failcb && failcb(0, obj.errcode? obj.errcode: 0, obj.errmsg? obj.errmsg: '');
            }
        }, function(errcode, errmsg){
            console.log('[readShareData]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    },

    dirtyCheck(str, successcb, failcb){
        this.urlbd.clear();
        this.urlbd.addKV('c', 'TextService')
        .addKV('a', 'dirtyWordCheck')
        .addKV('text', str);

        httpclient.httpGet(this.urlbd.baseurl, (restext)=>{
            var obj = httpclient.JSON_parse(restext);
            if(obj){
                console.log('[readShareData]success!'+JSON.stringify(obj));
                successcb && successcb(obj.errcode);
            }else{
                console.log('[readShareData]failed!');
                failcb && failcb(0, -1, restext);
            }
        }, function(errcode, errmsg){
            console.log('[readShareData]failed!'+errcode+":"+errmsg);
            failcb && failcb(errcode, 0, errmsg);
        });
    }
};

module.exports = jcshare;
