let commonutils = {
    //是否是全面屏
    isFringeScreen() {
        let isFringeScreen = false;
        let frameSize = cc.view.getFrameSize();
        if (cc.sys.os == cc.sys.OS_IOS) {
            isFringeScreen = (frameSize.width == 414 && frameSize.height == 896) || (frameSize.width == 375 && frameSize.height == 812);
        } else {
            if (frameSize.height / frameSize.width > (16 / 9)) {
                isFringeScreen = true
            }
        }
        return isFringeScreen;
    },

    clone(obj) {
        // Handle the 3 simple types, and null or undefined
        if (null == obj || "object" != typeof obj) return obj;

        // Handle Date
        if (obj instanceof Date) {
            var copy = new Date();
            copy.setTime(obj.getTime());
            return copy;
        }

        // Handle Array
        if (obj instanceof Array) {
            var copy = [];
            var len = obj.length;
            for (var i = 0; i < len; ++i) {
                copy[i] = this.clone(obj[i]);
            }
            return copy;
        }

        // Handle Object
        if (obj instanceof Object) {
            var copy = {};
            for (var attr in obj) {
                if (obj.hasOwnProperty(attr)) copy[attr] = this.clone(obj[attr]);
            }
            return copy;
        }

        throw new Error("Unable to copy obj! Its type isn't supported.");
    },

    cloneSimple(obj) {
        var copy = {};
        for (var attr in obj) {
            if (obj.hasOwnProperty(attr)) copy[attr] = obj[attr];
        }
        return copy;
    },

    cloneFull(obj) {
        var newObj = {};
        if (obj instanceof Array) {
            newObj = [];
        }
        for (var key in obj) {
            var val = obj[key];
            newObj[key] = typeof val === 'object' ? this.cloneFull(val) : val;
        }
        return newObj;
    },

    //[{weight}]
    /**
     * [计算权重]
     * @param  {[array]} obj_vec [对象数组，对象中要有权重变量weight]
     * @return {[number]}         [obj_vec数组索引]
     */
    calcWeight(obj_vec) {
        let all_weight = 0;
        let wvec = [];
        for (let obj of obj_vec) {
            all_weight += Math.max(0, obj.weight);
            wvec.push(all_weight);
        }
        if (all_weight <= 0) {
            return 0;
        }

        let ret = 0;
        let rand = Math.random() * all_weight;
        for (let i = 0; i < wvec.length; i++) {
            if (rand < wvec[i]) {
                ret = i;
                break;
            }
        }
        return ret;
    },

    calcRandNotRepet(obj_vec, count) {
        let ret_vec = [];
        if (obj_vec.length < count) {
            return obj_vec;
        }
        let obj_vec_tmp = this.clone(obj_vec);
        for (let i = 0; i < count; i++) {
            let rand_index = Math.floor(Math.random() * (obj_vec_tmp.length - i));
            let swap = this.clone(obj_vec_tmp[rand_index]);
            obj_vec_tmp[rand_index] = obj_vec_tmp[obj_vec_tmp.length - i - 1];
            obj_vec_tmp[obj_vec_tmp.length - i - 1] = swap;
            ret_vec.push(swap);
        }
        return ret_vec;
    },

    /**
     *
     * @param {Component} animation: cc.animation组件
     * @param {Array} frames : spriteframe对象列表
     * @param {Boolean} isloop : 是否循环播放
     * @param {number} sample : 每秒帧率，默认20
     */
    playAniFrames(animation, frames, isloop, sample){
        if(!animation || !frames){
            return;
        }
        let rfps = sample? sample: 20;
        let clip = cc.AnimationClip.createWithSpriteFrames(frames, rfps);
        clip.wrapMode = isloop? cc.WrapMode.Loop: cc.WrapMode.Default;
        animation.addClip(clip, 'anim_run');
        if(!animation.defaultClip){
            animation.defaultClip = clip;
        }
        animation.play('anim_run');
    },

        /**
     * 以某点为圆心，生成圆周上等分点的坐标
     *
     * @param {cc.Vec2} pos 圆心坐标
     * @param {number} r 半径
     * @param {number} count 等分点数量
     * @param {number} [randomScope=60] 等分点的随机波动范围
     * @returns {cc.Vec2[]} 返回等分点坐标
     */
    getCirclePoints(pos, r, count, randomScope)
    {
        let rscope = randomScope || 60;
        let points = [];
        let radians = (Math.PI / 180) * Math.round(360 / count);
        for (let i = 0; i < count; i++) {
            let x = pos.x + r * Math.sin(radians * i);
            let y = pos.y + r * Math.cos(radians * i);
            points.unshift(cc.v3(x + Math.random() * rscope, y + Math.random() * rscope, 0));
        }
        return points;
    },

    /**
     *
     * @param {cc.Prefab} pbCoin : coin预制体
     * @param {cc.Node} ndSrc : 生成coin的节点
     * @param {cc.Node} ndDst : coin飞到的节点
     * @param {number} count : 一次生成多少coin
     * @param {number} offset : coin离出生点偏移量
     * @param {cc.NodePool} pbPool : coin对象池
     */
    playCoinFlyAni(pbCoin, ndSrc, ndDst, count, offset, pbPool){
        let ndRoot = ndDst.parent;
        let endPt = ndDst.position;
        let startPt = ndRoot.convertToNodeSpaceAR(ndSrc.position);

        let ptlst = this.getCirclePoints(startPt, offset, count);

        let ndlst = [];
        for(let i =0;i<ptlst.count;i++){
            let pt = ptlst[i];
            let nd = pbPool? pbPool.get(): null;
            if(!nd){
                nd = cc.instantiate(pbCoin);
            }
            nd.setPosition(pt.x, pt.y);

            var a = endPt.x-pt.x;
            var b = endPt.y-pt.y;
            nd.dis = Math.sqrt(a*a+b*b);
            nd.startPoint = startPt;
            nd.endPoint = endPt;

            ndRoot.addChild(nd);
            ndlst.push(nd);
        }

        ndlst.sort(node => node.dis);

        ndlst.forEach((node, idx) => {
            node.runAction(cc.sequence(
                cc.moveTo(0.3, node.startPoint),
                cc.delayTime(idx * 0.03),
                cc.moveTo(0.6, node.endPoint),
                cc.callFunc(() => {
                    pbPool? pbPool.put(node): node.destory();
                })
            ))
        });
    }
};

module.exports = commonutils;
