/**
 * 日期和时间相关方法, 待扩充
 * */
let dateutils = {
  // 获取当天的日期, 返回'YYYY-MM-DD'
  getToday: function () {
    return new Date().toISOString().split('T')[0]
  },

  // 获取周一时间
  getFirstDay: function (millseconds) {
    var dt = millseconds? new Date(millseconds): new Date();
    var day = dt.getDay() || 7;
    var newdt = new Date(dt.getFullYear(), dt.getMonth(), dt.getDate() + 1 - day);
    newdt.setHours(0);
    newdt.setMinutes(0);
    newdt.setSeconds(0);
    newdt.setMilliseconds(0);
    return newdt;
  },

  // 时间转字符串
  formatDateStr: function(dt){
      return dt.getFullYear() + '-' + (dt.getMonth() + 1) + '-' + dt.getDate();
  },

  // 按固定格式转日期字符串
  dateFormatter: function(formatter, date){
    date = (date ? new Date(date) : new Date)
    const Y = date.getFullYear() + '',
             M = date.getMonth() + 1,
             D = date.getDate(),
             H = date.getHours(),
             m = date.getMinutes(),
             s = date.getSeconds()
       return formatter.replace(/YYYY|yyyy/g, Y)
              .replace(/YY|yy/g, Y.substr(2, 2))
              .replace(/MM/g, (M < 10 ? '0' : '') + M)
              .replace(/DD/g, (D < 10 ? '0' : '') + D)
              .replace(/HH|hh/g, (H < 10 ? '0' : '') + H)
              .replace(/mm/g, (m < 10 ? '0' : '') + m)
              .replace(/ss/g, (s < 10 ? '0' : '') + s)
   },

  // dtime距当前多少天
  DateGetDay(dtime){
      if(dtime == 0){
          return 0;
      }
      var d = new Date();
      var realdt = dtime - d.getTimezoneOffset()*60000;
      var oneDayTime = 1000 * 3600 * 24;
      return parseInt(realdt / oneDayTime);
  },

  // 是否是同一周
  isSameWeek: function(old, now) {
      var oldday = this.DateGetDay(old);
      var newday = this.DateGetDay(now);
      return parseInt((oldday + 3) / 7) == parseInt((newday + 3) / 7);
  },

  // 是否是同一天
  isSameDay: function(old, now){
      var oldday = this.DateGetDay(old);
      var newday = this.DateGetDay(now);
      return oldday == newday;
  },

  // 是否是昨天
  isSerialDay: function(olddt){
      var nowdt = new Date().getTime();
      var oldday = this.DateGetDay(olddt);
      var newday = this.DateGetDay(nowdt);
      return (newday - oldday) == 1;
  },

  // 是否是今天
  isNowDay: function(day){
      var now = new Date();
      return this.isSameDay(day, now);
  },

  	//间隔天数
	betweenDays(time1, time2) {
    	var v1 = Math.floor(time1/3600/24);
    	var v2 = Math.floor(time2/3600/24);
    	return Math.abs(v1 - v2);
	},
	//在当天的秒数
    getDaySeconds(time, incdays){
        return (Math.floor(time/1000/3600/24) + incdays)*3600*24;
    },
    getCurrMillSeconds() {
        return new Date().getTime();
    },
    getCurrSeconds() {
        var ms = this.getCurrMillSeconds();
        return Math.floor(ms/1000);
    },

    // 计算过期剩余秒数
    expiredSeconds(time_type, time_val, time_past){
        let tmnow = this.getCurrSeconds();
        let sec = 0;
        switch(time_type){
            case 1: {   //以当前时间为起点
                sec = tmnow - time_past;
            }break;
            case 2:{    //以当天0点时间为起点
                let tmsec = this.getDaySeconds(time_past*1000, 0);
                sec = tmnow - tmsec;
            }break;
            case 3:{    //以本周一0点时间为起点
                let fday = this.getFirstDay(time_past*1000);
                let tmsec = fday.getTime() * 1000;
                sec = tmnow - tmsec;
            }break;
            case 4:{    //以当月第一天0点为起点
                var lasttm = new Date(time_past*1000);
                var fday = new Date(lasttm.getFullYear(), lasttm.getMonth(), 1, 0, 0, 0, 0);
                let tmsec = fday.getTime() * 1000;
                sec = tmnow - tmsec;
            }break;
            case 5:{    //以下月第一天0点为起点
                var lasttm = new Date(time_past*1000);
                let y = lasttm.getFullYear();
                let m = lasttm.getMonth();
                if(m == 11){
                    m = 0;
                    y++;
                }else{
                    m++;
                }
                var fday = new Date(y, m, 1, 0, 0, 0, 0);
                let tmsec = fday.getTime() * 1000;
                sec = tmnow - tmsec;
            }break;
        }
        return time_val - sec;
    },

  /**
   * 将秒格式化成 天:小时:分:秒
   * @param {number} countTime 要格式化的秒
   * @param {number} showNum   要显示几个段, 比例 2, 则显示 x分: x秒, 3: x小时:x分:x秒
   * */
  formatSeconds(countTime, showNum){
    let str = "";
    countTime = countTime < 0 ? 0 : countTime;
    let day = Math.floor(countTime / (60 * 60 * 24));//天
    let hours = Math.floor((countTime - day *(60 * 60 * 24))/ (60 * 60)) ; //小时
    let min = Math.floor((countTime - day *(60 * 60 * 24) -  hours*(60 *60)) / 60);
    let sec = Math.floor((countTime - day *(60 * 60 * 24) -  hours*(60 *60) - min * 60));
    let timeList = [sec,min,hours,day];
    let timeMaxList = [60,60,24,99];

    let formatToStr = function (_index) {
      let num = 0;
      if(_index === showNum-1 && showNum < 4) {
        //最后一个且还有更高单位，合并
        for(let i = _index+1; i < 4; i++){
          if(timeList[i] > 0) {
            let tempNum = timeList[i];
            for(let j = i; j > _index; j--){
              tempNum = tempNum*timeMaxList[j-1];
            }
            num += tempNum;
          }
        }
        num += timeList[_index];
      }
      else {
        num = timeList[_index];
      }
      num = '' + num;
      return num.padStart(2, '0');
    };

    for(let i = showNum-1; i >= 0; i--){
      let curStr = formatToStr(i);
      str += curStr;
      if(i !== 0) {
        str += ":";
      }
    }
    return str;
  },
}

module.exports = dateutils;
